# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 Red Hat, Inc.
# Copyright (c) 2010 Ville Skyttä
# Copyright (c) 2009 Tim Lauridsen
# Copyright (c) 2007 Marcus Kuhn
#
# kitchen is free software; you can redistribute it and/or modify it under the
# terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 2.1 of the License, or (at your option)
# any later version.
#
# kitchen is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
# more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with kitchen; if not, see <http://www.gnu.org/licenses/>
#
# Authors:
#   James Antill <james@fedoraproject.org>
#   Marcus Kuhn
#   Toshio Kuratomi <toshio@fedoraproject.org>
#   Tim Lauridsen
#   Ville Skyttä
#
# Portions of this are from yum/i18n.py
'''
-----------------------
Format Text for Display
-----------------------

Functions related to displaying unicode text.  Unicode characters don't all
have the same width so we need helper functions for displaying them.

.. versionadded:: 0.2 kitchen.display API 1.0.0
'''
import itertools
import unicodedata

from kitchen.text.converters import to_unicode, to_bytes
from kitchen.text.exceptions import ControlCharError

# This is ported from ustr_utf8_* which I got from:
#     http://www.cl.cam.ac.uk/~mgk25/ucs/wcwidth.c
#  I've tried to leave it close to the original C (same names etc.) so that
# it is easy to read/compare both versions... James Antilles

#
# Reimplemented quite a bit of this for speed.  Use the bzr log or annotate
# commands to see what I've changed since importing this file.-Toshio Kuratomi

# ----------------------------- BEG utf8 ------------------to-----------
# This is an implementation of wcwidth() and wcswidth() (defined in
# IEEE Std 1002.1-2001) for Unicode.
#
# http://www.opengroup.org/onlinepubs/007904975/functions/wcwidth.html
# http://www.opengroup.org/onlinepubs/007904975/functions/wcswidth.html
#
# In fixed-width output devices, Latin characters all occupy a single
# "cell" position of equal width, whereas ideographic CJK characters
# occupy two such cells. Interoperability between terminal-line
# applications and (teletype-style) character terminals using the
# UTF-8 encoding requires agreement on which character should advance
# the cursor by how many cell positions. No established formal
# standards exist at present on which Unicode character shall occupy
# how many cell positions on character terminals. These routines are
# a first attempt of defining such behavior based on simple rules
# applied to data provided by the Unicode Consortium.
#
# [...]
#
# Markus Kuhn -- 2007-05-26 (Unicode 5.0)
#
# Permission to use, copy, modify, and distribute this software
# for any purpose and without fee is hereby granted. The author
# disclaims all warranties with regard to this software.
#
# Latest version: http://www.cl.cam.ac.uk/~mgk25/ucs/wcwidth.c

# Renamed but still pretty much JA's port of MK's code
def _interval_bisearch(value, table):
    '''Binary search in an interval table.

    :arg value: numeric value to search for
    :arg table: Ordered list of intervals.  This is a list of two-tuples.  The
        elements of the two-tuple define an interval's start and end points.
    :returns: If :attr:`value` is found within an interval in the :attr:`table`
        return :data:`True`.  Otherwise, :data:`False`

    This function checks whether a numeric value is present within a table
    of intervals.  It checks using a binary search algorithm, dividing the
    list of values in half and checking against the values until it determines
    whether the value is in the table.
    '''
    minimum = 0
    maximum = len(table) - 1
    if value < table[minimum][0] or value > table[maximum][1]:
        return False

    while maximum >= minimum:
        mid = divmod(minimum + maximum, 2)[0]
        if value > table[mid][1]:
            minimum = mid + 1
        elif value < table[mid][0]:
            maximum = mid - 1
        else:
            return True

    return False

_COMBINING = (
        (0x300, 0x36f), (0x483, 0x489), (0x591, 0x5bd),
        (0x5bf, 0x5bf), (0x5c1, 0x5c2), (0x5c4, 0x5c5),
        (0x5c7, 0x5c7), (0x600, 0x603), (0x610, 0x61a),
        (0x64b, 0x65f), (0x670, 0x670), (0x6d6, 0x6e4),
        (0x6e7, 0x6e8), (0x6ea, 0x6ed), (0x70f, 0x70f),
        (0x711, 0x711), (0x730, 0x74a), (0x7a6, 0x7b0),
        (0x7eb, 0x7f3), (0x7fd, 0x7fd), (0x816, 0x819),
        (0x81b, 0x823), (0x825, 0x827), (0x829, 0x82d),
        (0x859, 0x85b), (0x8d3, 0x8e1), (0x8e3, 0x8ff),
        (0x901, 0x902), (0x93c, 0x93c), (0x941, 0x948),
        (0x94d, 0x94d), (0x951, 0x954), (0x962, 0x963),
        (0x981, 0x981), (0x9bc, 0x9bc), (0x9c1, 0x9c4),
        (0x9cd, 0x9cd), (0x9e2, 0x9e3), (0x9fe, 0x9fe),
        (0xa01, 0xa02), (0xa3c, 0xa3c), (0xa41, 0xa42),
        (0xa47, 0xa48), (0xa4b, 0xa4d), (0xa70, 0xa71),
        (0xa81, 0xa82), (0xabc, 0xabc), (0xac1, 0xac5),
        (0xac7, 0xac8), (0xacd, 0xacd), (0xae2, 0xae3),
        (0xb01, 0xb01), (0xb3c, 0xb3c), (0xb3f, 0xb3f),
        (0xb41, 0xb43), (0xb4d, 0xb4d), (0xb56, 0xb56),
        (0xb82, 0xb82), (0xbc0, 0xbc0), (0xbcd, 0xbcd),
        (0xc3e, 0xc40), (0xc46, 0xc48), (0xc4a, 0xc4d),
        (0xc55, 0xc56), (0xcbc, 0xcbc), (0xcbf, 0xcbf),
        (0xcc6, 0xcc6), (0xccc, 0xccd), (0xce2, 0xce3),
        (0xd3b, 0xd3c), (0xd41, 0xd43), (0xd4d, 0xd4d),
        (0xdca, 0xdca), (0xdd2, 0xdd4), (0xdd6, 0xdd6),
        (0xe31, 0xe31), (0xe34, 0xe3a), (0xe47, 0xe4e),
        (0xeb1, 0xeb1), (0xeb4, 0xebc), (0xec8, 0xecd),
        (0xf18, 0xf19), (0xf35, 0xf35), (0xf37, 0xf37),
        (0xf39, 0xf39), (0xf71, 0xf7e), (0xf80, 0xf84),
        (0xf86, 0xf87), (0xf90, 0xf97), (0xf99, 0xfbc),
        (0xfc6, 0xfc6), (0x102d, 0x1030), (0x1032, 0x1032),
        (0x1036, 0x1037), (0x1039, 0x103a), (0x1058, 0x1059),
        (0x108d, 0x108d), (0x1160, 0x11ff), (0x135d, 0x135f),
        (0x1712, 0x1714), (0x1732, 0x1734), (0x1752, 0x1753),
        (0x1772, 0x1773), (0x17b4, 0x17b5), (0x17b7, 0x17bd),
        (0x17c6, 0x17c6), (0x17c9, 0x17d3), (0x17dd, 0x17dd),
        (0x180b, 0x180d), (0x18a9, 0x18a9), (0x1920, 0x1922),
        (0x1927, 0x1928), (0x1932, 0x1932), (0x1939, 0x193b),
        (0x1a17, 0x1a18), (0x1a60, 0x1a60), (0x1a75, 0x1a7c),
        (0x1a7f, 0x1a7f), (0x1ab0, 0x1abd), (0x1b00, 0x1b03),
        (0x1b34, 0x1b34), (0x1b36, 0x1b3a), (0x1b3c, 0x1b3c),
        (0x1b42, 0x1b42), (0x1b44, 0x1b44), (0x1b6b, 0x1b73),
        (0x1baa, 0x1bab), (0x1be6, 0x1be6), (0x1bf2, 0x1bf3),
        (0x1c37, 0x1c37), (0x1cd0, 0x1cd2), (0x1cd4, 0x1ce0),
        (0x1ce2, 0x1ce8), (0x1ced, 0x1ced), (0x1cf4, 0x1cf4),
        (0x1cf8, 0x1cf9), (0x1dc0, 0x1df9), (0x1dfb, 0x1dff),
        (0x200b, 0x200f), (0x202a, 0x202e), (0x2060, 0x2063),
        (0x206a, 0x206f), (0x20d0, 0x20f0), (0x2cef, 0x2cf1),
        (0x2d7f, 0x2d7f), (0x2de0, 0x2dff), (0x302a, 0x302f),
        (0x3099, 0x309a), (0xa66f, 0xa66f), (0xa674, 0xa67d),
        (0xa69e, 0xa69f), (0xa6f0, 0xa6f1), (0xa806, 0xa806),
        (0xa80b, 0xa80b), (0xa825, 0xa826), (0xa8c4, 0xa8c4),
        (0xa8e0, 0xa8f1), (0xa92b, 0xa92d), (0xa953, 0xa953),
        (0xa9b3, 0xa9b3), (0xa9c0, 0xa9c0), (0xaab0, 0xaab0),
        (0xaab2, 0xaab4), (0xaab7, 0xaab8), (0xaabe, 0xaabf),
        (0xaac1, 0xaac1), (0xaaf6, 0xaaf6), (0xabed, 0xabed),
        (0xfb1e, 0xfb1e), (0xfe00, 0xfe0f), (0xfe20, 0xfe2f),
        (0xfeff, 0xfeff), (0xfff9, 0xfffb), (0x101fd, 0x101fd),
        (0x102e0, 0x102e0), (0x10376, 0x1037a), (0x10a01, 0x10a03),
        (0x10a05, 0x10a06), (0x10a0c, 0x10a0f), (0x10a38, 0x10a3a),
        (0x10a3f, 0x10a3f), (0x10ae5, 0x10ae6), (0x10d24, 0x10d27),
        (0x10f46, 0x10f50), (0x11046, 0x11046), (0x1107f, 0x1107f),
        (0x110b9, 0x110ba), (0x11100, 0x11102), (0x11133, 0x11134),
        (0x11173, 0x11173), (0x111c0, 0x111c0), (0x111ca, 0x111ca),
        (0x11235, 0x11236), (0x112e9, 0x112ea), (0x1133b, 0x1133c),
        (0x1134d, 0x1134d), (0x11366, 0x1136c), (0x11370, 0x11374),
        (0x11442, 0x11442), (0x11446, 0x11446), (0x1145e, 0x1145e),
        (0x114c2, 0x114c3), (0x115bf, 0x115c0), (0x1163f, 0x1163f),
        (0x116b6, 0x116b7), (0x1172b, 0x1172b), (0x11839, 0x1183a),
        (0x119e0, 0x119e0), (0x11a34, 0x11a34), (0x11a47, 0x11a47),
        (0x11a99, 0x11a99), (0x11c3f, 0x11c3f), (0x11d42, 0x11d42),
        (0x11d44, 0x11d45), (0x11d97, 0x11d97), (0x16af0, 0x16af4),
        (0x16b30, 0x16b36), (0x1bc9e, 0x1bc9e), (0x1d165, 0x1d169),
        (0x1d16d, 0x1d182), (0x1d185, 0x1d18b), (0x1d1aa, 0x1d1ad),
        (0x1d242, 0x1d244), (0x1e000, 0x1e006), (0x1e008, 0x1e018),
        (0x1e01b, 0x1e021), (0x1e023, 0x1e024), (0x1e026, 0x1e02a),
        (0x1e130, 0x1e136), (0x1e2ec, 0x1e2ef), (0x1e8d0, 0x1e8d6),
        (0x1e944, 0x1e94a), (0xe0001, 0xe0001), (0xe0020, 0xe007f),
        (0xe0100, 0xe01ef), )

'''
Internal table, provided by this module to list :term:`code points` which
combine with other characters and therefore should have no :term:`textual
width`.  This is a sorted :class:`tuple` of non-overlapping intervals.  Each
interval is a :class:`tuple` listing a starting :term:`code point` and ending
:term:`code point`.  Every :term:`code point` between the two end points is
a combining character.

.. seealso::

    :func:`~kitchen.text.display._generate_combining_table`
        for how this table is generated

This table was last regenerated on python-3.8.0a3 with
:data:`unicodedata.unidata_version` 12.0.0
'''
# New function from Toshio Kuratomi (LGPLv2+)
def _generate_combining_table():
    '''Combine Markus Kuhn's data with :mod:`unicodedata` to make combining
    char list

    :rtype: :class:`tuple` of tuples
    :returns: :class:`tuple` of intervals of :term:`code points` that are
        combining character.  Each interval is a 2-:class:`tuple` of the
        starting :term:`code point` and the ending :term:`code point` for the
        combining characters.

    In normal use, this function serves to tell how we're generating the
    combining char list.  For speed reasons, we use this to generate a static
    list and just use that later.

    Markus Kuhn's list of combining characters is more complete than what's in
    the python :mod:`unicodedata` library but the python :mod:`unicodedata` is
    synced against later versions of the unicode database

    This is used to generate the :data:`~kitchen.text.display._COMBINING`
    table.
    '''
    # Marcus Kuhn's sorted list of non-overlapping intervals of non-spacing
    # characters generated ifrom Unicode 5.0 data by:
    # "uniset +cat=Me +cat=Mn +cat=Cf -00AD +1160-11FF +200B c"
    markus_kuhn_combining_5_0 = (
        (0x0300, 0x036F), (0x0483, 0x0486), (0x0488, 0x0489),
        (0x0591, 0x05BD), (0x05BF, 0x05BF), (0x05C1, 0x05C2),
        (0x05C4, 0x05C5), (0x05C7, 0x05C7), (0x0600, 0x0603),
        (0x0610, 0x0615), (0x064B, 0x065E), (0x0670, 0x0670),
        (0x06D6, 0x06E4), (0x06E7, 0x06E8), (0x06EA, 0x06ED),
        (0x070F, 0x070F), (0x0711, 0x0711), (0x0730, 0x074A),
        (0x07A6, 0x07B0), (0x07EB, 0x07F3), (0x0901, 0x0902),
        (0x093C, 0x093C), (0x0941, 0x0948), (0x094D, 0x094D),
        (0x0951, 0x0954), (0x0962, 0x0963), (0x0981, 0x0981),
        (0x09BC, 0x09BC), (0x09C1, 0x09C4), (0x09CD, 0x09CD),
        (0x09E2, 0x09E3), (0x0A01, 0x0A02), (0x0A3C, 0x0A3C),
        (0x0A41, 0x0A42), (0x0A47, 0x0A48), (0x0A4B, 0x0A4D),
        (0x0A70, 0x0A71), (0x0A81, 0x0A82), (0x0ABC, 0x0ABC),
        (0x0AC1, 0x0AC5), (0x0AC7, 0x0AC8), (0x0ACD, 0x0ACD),
        (0x0AE2, 0x0AE3), (0x0B01, 0x0B01), (0x0B3C, 0x0B3C),
        (0x0B3F, 0x0B3F), (0x0B41, 0x0B43), (0x0B4D, 0x0B4D),
        (0x0B56, 0x0B56), (0x0B82, 0x0B82), (0x0BC0, 0x0BC0),
        (0x0BCD, 0x0BCD), (0x0C3E, 0x0C40), (0x0C46, 0x0C48),
        (0x0C4A, 0x0C4D), (0x0C55, 0x0C56), (0x0CBC, 0x0CBC),
        (0x0CBF, 0x0CBF), (0x0CC6, 0x0CC6), (0x0CCC, 0x0CCD),
        (0x0CE2, 0x0CE3), (0x0D41, 0x0D43), (0x0D4D, 0x0D4D),
        (0x0DCA, 0x0DCA), (0x0DD2, 0x0DD4), (0x0DD6, 0x0DD6),
        (0x0E31, 0x0E31), (0x0E34, 0x0E3A), (0x0E47, 0x0E4E),
        (0x0EB1, 0x0EB1), (0x0EB4, 0x0EB9), (0x0EBB, 0x0EBC),
        (0x0EC8, 0x0ECD), (0x0F18, 0x0F19), (0x0F35, 0x0F35),
        (0x0F37, 0x0F37), (0x0F39, 0x0F39), (0x0F71, 0x0F7E),
        (0x0F80, 0x0F84), (0x0F86, 0x0F87), (0x0F90, 0x0F97),
        (0x0F99, 0x0FBC), (0x0FC6, 0x0FC6), (0x102D, 0x1030),
        (0x1032, 0x1032), (0x1036, 0x1037), (0x1039, 0x1039),
        (0x1058, 0x1059), (0x1160, 0x11FF), (0x135F, 0x135F),
        (0x1712, 0x1714), (0x1732, 0x1734), (0x1752, 0x1753),
        (0x1772, 0x1773), (0x17B4, 0x17B5), (0x17B7, 0x17BD),
        (0x17C6, 0x17C6), (0x17C9, 0x17D3), (0x17DD, 0x17DD),
        (0x180B, 0x180D), (0x18A9, 0x18A9), (0x1920, 0x1922),
        (0x1927, 0x1928), (0x1932, 0x1932), (0x1939, 0x193B),
        (0x1A17, 0x1A18), (0x1B00, 0x1B03), (0x1B34, 0x1B34),
        (0x1B36, 0x1B3A), (0x1B3C, 0x1B3C), (0x1B42, 0x1B42),
        (0x1B6B, 0x1B73), (0x1DC0, 0x1DCA), (0x1DFE, 0x1DFF),
        (0x200B, 0x200F), (0x202A, 0x202E), (0x2060, 0x2063),
        (0x206A, 0x206F), (0x20D0, 0x20EF), (0x302A, 0x302F),
        (0x3099, 0x309A), (0xA806, 0xA806), (0xA80B, 0xA80B),
        (0xA825, 0xA826), (0xFB1E, 0xFB1E), (0xFE00, 0xFE0F),
        (0xFE20, 0xFE23), (0xFEFF, 0xFEFF), (0xFFF9, 0xFFFB),
        (0x10A01, 0x10A03), (0x10A05, 0x10A06), (0x10A0C, 0x10A0F),
        (0x10A38, 0x10A3A), (0x10A3F, 0x10A3F), (0x1D167, 0x1D169),
        (0x1D173, 0x1D182), (0x1D185, 0x1D18B), (0x1D1AA, 0x1D1AD),
        (0x1D242, 0x1D244), (0xE0001, 0xE0001), (0xE0020, 0xE007F),
        (0xE0100, 0xE01EF))
    combining = []
    in_interval = False
    interval = []
    for codepoint in range(0, 0xFFFFF + 1):
        if _interval_bisearch(codepoint, markus_kuhn_combining_5_0) or \
                unicodedata.combining(chr(codepoint)):
            if not in_interval:
                # Found first part of an interval
                interval = [codepoint]
                in_interval = True
        else:
            if in_interval:
                in_interval = False
                interval.append(codepoint - 1)
                combining.append(interval)

    if in_interval:
        # If we're at the end and the interval is open, close it.
        # :W0631: We looped through a static range so we know codepoint is
        #   defined here
        #pylint:disable-msg=W0631
        interval.append(codepoint)
        combining.append(interval)

    return tuple(map(tuple, combining))

# New function from Toshio Kuratomi (LGPLv2+)
def _print_combining_table():
    '''Print out a new :data:`_COMBINING` table

    This will print a new :data:`_COMBINING` table in the format used in
    :file:`kitchen/text/display.py`.  It's useful for updating the
    :data:`_COMBINING` table with updated data from a new python as the format
    won't change from what's already in the file.
    '''
    table = _generate_combining_table()
    entries = 0
    print('_COMBINING = (')
    for pair in table:
        if entries >= 3:
            entries = 0
            print()
        if entries == 0:
            print('       ', end=' ')
        entries += 1
        entry = '(0x%x, 0x%x),' % pair
        print(entry, end=' ')
    print(')')

# Handling of control chars rewritten.  Rest is JA's port of MK's C code.
# -Toshio Kuratomi
def _ucp_width(ucs, control_chars='guess'):
    '''Get the :term:`textual width` of a ucs character

    :arg ucs: integer representing a single unicode :term:`code point`
    :kwarg control_chars: specify how to deal with :term:`control characters`.
        Possible values are:

        :guess: (default) will take a guess for :term:`control character`
            widths.  Most codes will return zero width.  ``backspace``,
            ``delete``, and ``clear delete`` return -1.  ``escape`` currently
            returns -1 as well but this is not guaranteed as it's not always
            correct
        :strict: will raise :exc:`~kitchen.text.exceptions.ControlCharError`
            if a :term:`control character` is encountered

    :raises ControlCharError: if the :term:`code point` is a unicode
        :term:`control character` and :attr:`control_chars` is set to 'strict'
    :returns: :term:`textual width` of the character.

    .. note::

        It's important to remember this is :term:`textual width` and not the
        number of characters or bytes.
    '''
    # test for 8-bit control characters
    if ucs < 32 or (ucs < 0xa0 and ucs >= 0x7f):
        # Control character detected
        if control_chars == 'strict':
            raise ControlCharError('_ucp_width does not understand how to'
                ' assign a width value to control characters.')
        if ucs in (0x08, 0x07F, 0x94):
            # Backspace, delete, and clear delete remove a single character
            return -1
        if ucs == 0x1b:
            # Excape is tricky.  It removes some number of characters that
            # come after it but the amount is dependent on what is
            # interpreting the code.
            # So this is going to often be wrong but other values will be
            # wrong as well.
            return -1
        # All other control characters get 0 width
        return 0

    if _interval_bisearch(ucs, _COMBINING):
        # Combining characters return 0 width as they will be combined with
        # the width from other characters
        return 0

    # if we arrive here, ucs is not a combining or C0/C1 control character

    return (1 +
      (ucs >= 0x1100 and
       (ucs <= 0x115f or                     # Hangul Jamo init. consonants
        ucs == 0x2329 or ucs == 0x232a or
        (ucs >= 0x2e80 and ucs <= 0xa4cf and
         ucs != 0x303f) or                   # CJK ... Yi
        (ucs >= 0xac00 and ucs <= 0xd7a3) or # Hangul Syllables
        (ucs >= 0xf900 and ucs <= 0xfaff) or # CJK Compatibility Ideographs
        (ucs >= 0xfe10 and ucs <= 0xfe19) or # Vertical forms
        (ucs >= 0xfe30 and ucs <= 0xfe6f) or # CJK Compatibility Forms
        (ucs >= 0xff00 and ucs <= 0xff60) or # Fullwidth Forms
        (ucs >= 0xffe0 and ucs <= 0xffe6) or
        (ucs >= 0x20000 and ucs <= 0x2fffd) or
        (ucs >= 0x30000 and ucs <= 0x3fffd))))

# Wholly rewritten by me (LGPLv2+) -Toshio Kuratomi
def textual_width(msg, control_chars='guess', encoding='utf-8',
        errors='replace'):
    '''Get the :term:`textual width` of a string

    :arg msg: :class:`str` string or byte :class:`bytes` to get the width of
    :kwarg control_chars: specify how to deal with :term:`control characters`.
        Possible values are:

        :guess: (default) will take a guess for :term:`control character`
            widths.  Most codes will return zero width.  ``backspace``,
            ``delete``, and ``clear delete`` return -1.  ``escape`` currently
            returns -1 as well but this is not guaranteed as it's not always
            correct
        :strict: will raise :exc:`kitchen.text.exceptions.ControlCharError`
            if a :term:`control character` is encountered

    :kwarg encoding: If we are given a byte :class:`bytes` this is used to
        decode it into :class:`str` string.  Any characters that are not
        decodable in this encoding will get a value dependent on the
        :attr:`errors` parameter.
    :kwarg errors: How to treat errors encoding the byte :class:`bytes` to
        :class:`str` string.  Legal values are the same as for
        :func:`kitchen.text.converters.to_unicode`.  The default value of
        ``replace`` will cause undecodable byte sequences to have a width of
        one. ``ignore`` will have a width of zero.
    :raises ControlCharError: if :attr:`msg` contains a :term:`control
        character` and :attr:`control_chars` is ``strict``.
    :returns: :term:`Textual width` of the :attr:`msg`.  This is the amount of
        space that the string will consume on a monospace display.  It's
        measured in the number of cell positions or columns it will take up on
        a monospace display.  This is **not** the number of glyphs that are in
        the string.

    .. note::

        This function can be wrong sometimes because Unicode does not specify
        a strict width value for all of the :term:`code points`.  In
        particular, we've found that some Tamil characters take up to four
        character cells but we return a lesser amount.
    '''
    # On python 2.6.4, x86_64, I've benchmarked a few alternate
    # implementations::
    #
    #   timeit.repeat('display.textual_width(data)',
    #       'from __main__ import display, data', number=100)
    # I varied data by size and content (1MB of ascii, a few words, 43K utf8,
    # unicode type
    #
    # :this implementation: fastest across the board
    #
    # :list comprehension: 6-16% slower
    #   return sum([_ucp_width(ord(c), control_chars=control_chars)
    #       for c in msg])
    #
    # :generator expression: 9-18% slower
    #   return sum((_ucp_width(ord(c), control_chars=control_chars) for c in
    #           msg))
    #
    # :lambda: 10-19% slower
    #   return sum(itertools.imap(lambda x: _ucp_width(ord(x), control_chars),
    #           msg))
    #
    # :partial application: 13-22% slower
    #   func = functools.partial(_ucp_width, control_chars=control_chars)
    #   return sum(itertools.imap(func, itertools.imap(ord, msg)))
    #
    # :the original code: 4-38% slower
    #   The 4% was for the short, ascii only string.  All the other pieces of
    #   data yielded over 30% slower times.

    # Non decodable data is just assigned a single cell width
    msg = to_unicode(msg, encoding=encoding, errors=errors)
    # Add the width of each char
    return sum(
            # calculate width of each char
            itertools.starmap(_ucp_width,
                # Setup the arguments to _ucp_width
                zip(
                    # int value of each char
                    map(ord, msg),
                    # control_chars arg in a form that izip will deal with
                    itertools.repeat(control_chars))))

# Wholly rewritten by me -Toshio Kuratomi
def textual_width_chop(msg, chop, encoding='utf-8', errors='replace'):
    '''Given a string, return it chopped to a given :term:`textual width`

    :arg msg: :class:`str` string or byte :class:`bytes` to chop
    :arg chop: Chop :attr:`msg` if it exceeds this :term:`textual width`
    :kwarg encoding: If we are given a byte :class:`bytes`, this is used to
        decode it into a :class:`str` string.  Any characters that are not
        decodable in this encoding will be assigned a width of one.
    :kwarg errors: How to treat errors encoding the byte :class:`bytes` to
        :class:`str`.  Legal values are the same as for
        :func:`kitchen.text.converters.to_unicode`
    :rtype: :class:`str` string
    :returns: :class:`str` string of the :attr:`msg` chopped at the given
        :term:`textual width`

    This is what you want to use instead of ``%.*s``, as it does the "right"
    thing with regard to :term:`UTF-8` sequences, :term:`control characters`,
    and characters that take more than one cell position. Eg::

        >>> # Wrong: only displays 8 characters because it is operating on bytes
        >>> print "%.*s" % (10, 'café ñunru!')
        café ñun
        >>> # Properly operates on graphemes
        >>> '%s' % (textual_width_chop('café ñunru!', 10))
        café ñunru
        >>> # takes too many columns because the kanji need two cell positions
        >>> print '1234567890\\n%.*s' % (10, u'一二三四五六七八九十')
        1234567890
        一二三四五六七八九十
        >>> # Properly chops at 10 columns
        >>> print '1234567890\\n%s' % (textual_width_chop(u'一二三四五六七八九十', 10))
        1234567890
        一二三四五

    '''

    msg = to_unicode(msg, encoding=encoding, errors=errors)

    width = textual_width(msg)
    if width <= chop:
        return msg
    maximum = len(msg)
    if maximum > chop * 2:
        # A character can take at most 2 cell positions so this is the actual
        # maximum
        maximum = chop * 2
    minimum = 0
    eos = maximum
    if eos > chop:
        eos = chop
    width = textual_width(msg[:eos])

    while True:
        # if current width is high,
        if width > chop:
            # calculate new midpoint
            mid = minimum + (eos - minimum) // 2
            if mid == eos:
                break
            if (eos - chop) < (eos - mid):
                while width > chop:
                    width = width - _ucp_width(ord(msg[eos-1]))
                    eos -= 1
                return msg[:eos]
            # subtract distance between eos and mid from width
            width = width - textual_width(msg[mid:eos])
            maximum = eos
            eos = mid
        # if current width is low,
        elif width < chop:
            # Note: at present, the if (eos - chop) < (eos - mid):
            # short-circuit above means that we never use this branch.

            # calculate new midpoint
            mid = eos + (maximum - eos) // 2
            if mid == eos:
                break
            if (chop - eos) < (mid - eos):
                while width < chop:
                    new_width = _ucp_width(ord(msg[eos]))
                    width = width + new_width
                    eos += 1
                return msg[:eos]

            # add distance between eos and new mid to width
            width = width + textual_width(msg[eos:mid])
            minimum = eos
            eos = mid
            if eos > maximum:
                eos = maximum
                break
        # if current is just right
        else:
            return msg[:eos]
    return msg[:eos]

# I made some adjustments for using unicode but largely unchanged from JA's
# port of MK's code -Toshio
def textual_width_fill(msg, fill, chop=None, left=True, prefix='', suffix=''):
    '''Expand a :class:`str` string to a specified :term:`textual width`
    or chop to same

    :arg msg: :class:`str` string to format
    :arg fill: pad string until the :term:`textual width` of the string is
        this length
    :kwarg chop: before doing anything else, chop the string to this length.
        Default: Don't chop the string at all
    :kwarg left: If :data:`True` (default) left justify the string and put the
        padding on the right.  If :data:`False`, pad on the left side.
    :kwarg prefix: Attach this string before the field we're filling
    :kwarg suffix: Append this string to the end of the field we're filling
    :rtype: :class:`str` string
    :returns: :attr:`msg` formatted to fill the specified width.  If no
        :attr:`chop` is specified, the string could exceed the fill length
        when completed.  If :attr:`prefix` or :attr:`suffix` are printable
        characters, the string could be longer than the fill width.

    .. note::

        :attr:`prefix` and :attr:`suffix` should be used for "invisible"
        characters like highlighting, color changing escape codes, etc.  The
        fill characters are appended outside of any :attr:`prefix` or
        :attr:`suffix` elements.  This allows you to only highlight
        :attr:`msg` inside of the field you're filling.

    .. warning::

        :attr:`msg`, :attr:`prefix`, and :attr:`suffix` should all be
        representable as unicode characters.  In particular, any escape
        sequences in :attr:`prefix` and :attr:`suffix` need to be convertible
        to :class:`str`.  If you need to use byte sequences here rather
        than unicode characters, use
        :func:`~kitchen.text.display.byte_string_textual_width_fill` instead.

    This function expands a string to fill a field of a particular
    :term:`textual width`.  Use it instead of ``%*.*s``, as it does the
    "right" thing with regard to :term:`UTF-8` sequences, :term:`control
    characters`, and characters that take more than one cell position in
    a display.  Example usage::

        >>> msg = u'一二三四五六七八九十'
        >>> # Wrong: This uses 10 characters instead of 10 cells:
        >>> u":%-*.*s:" % (10, 10, msg[:9])
        :一二三四五六七八九 :
        >>> # This uses 10 cells like we really want:
        >>> u":%s:" % (textual_width_fill(msg[:9], 10, 10))
        :一二三四五:

        >>> # Wrong: Right aligned in the field, but too many cells
        >>> u"%20.10s" % (msg)
                  一二三四五六七八九十
        >>> # Correct: Right aligned with proper number of cells
        >>> u"%s" % (textual_width_fill(msg, 20, 10, left=False))
                  一二三四五

        >>> # Wrong: Adding some escape characters to highlight the line but too many cells
        >>> u"%s%20.10s%s" % (prefix, msg, suffix)
        u'\x1b[7m          一二三四五六七八九十\x1b[0m'
        >>> # Correct highlight of the line
        >>> u"%s%s%s" % (prefix, display.textual_width_fill(msg, 20, 10, left=False), suffix)
        u'\x1b[7m          一二三四五\x1b[0m'

        >>> # Correct way to not highlight the fill
        >>> u"%s" % (display.textual_width_fill(msg, 20, 10, left=False, prefix=prefix, suffix=suffix))
        u'          \x1b[7m一二三四五\x1b[0m'
    '''
    msg = to_unicode(msg)
    if chop is not None:
        msg = textual_width_chop(msg, chop)
    width = textual_width(msg)
    if width >= fill:
        if prefix or suffix:
            msg = ''.join([prefix, msg, suffix])
    else:
        extra = ' ' * (fill - width)
        if left:
            msg = ''.join([prefix, msg, suffix, extra])
        else:
            msg = ''.join([extra, prefix, msg, suffix])
    return msg

def _textual_width_le(width, *args):
    '''Optimize the common case when deciding which :term:`textual width` is
    larger

    :arg width: :term:`textual width` to compare against.
    :arg \*args: :class:`str` strings to check the total :term:`textual
        width` of
    :returns: :data:`True` if the total length of :attr:`args` are less than
        or equal to :attr:`width`.  Otherwise :data:`False`.

    We often want to know "does X fit in Y".  It takes a while to use
    :func:`textual_width` to calculate this.  However, we know that the number
    of canonically composed :class:`str` characters is always going to
    have 1 or 2 for the :term:`textual width` per character.  With this we can
    take the following shortcuts:

    1) If the number of canonically composed characters is more than width,
       the true :term:`textual width` cannot be less than width.
    2) If the number of canonically composed characters * 2 is less than the
       width then the :term:`textual width` must be ok.

    :term:`textual width` of a canonically composed :class:`str` string
    will always be greater than or equal to the the number of :class:`str`
    characters.  So we can first check if the number of composed
    :class:`str` characters is less than the asked for width.  If it is we
    can return :data:`True` immediately.  If not, then we must do a full
    :term:`textual width` lookup.
    '''
    string = ''.join(args)
    string = unicodedata.normalize('NFC', string)
    if len(string) > width:
        return False
    elif len(string) * 2 <= width:
        return True
    elif len(to_bytes(string)) <= width:
        # Check against bytes.
        # utf8 has the property of having the same amount or more bytes per
        # character than textual width.
        return True
    else:
        true_width = textual_width(string)
    return true_width <= width

def wrap(text, width=70, initial_indent='', subsequent_indent='',
        encoding='utf-8', errors='replace'):
    '''Works like we want :func:`textwrap.wrap` to work,

    :arg text: :class:`str` string or byte :class:`bytes` to wrap
    :kwarg width: :term:`textual width` at which to wrap.  Default: 70
    :kwarg initial_indent: string to use to indent the first line.  Default:
        do not indent.
    :kwarg subsequent_indent: string to use to wrap subsequent lines.
        Default: do not indent
    :kwarg encoding: Encoding to use if :attr:`text` is a byte :class:`bytes`
    :kwarg errors: error handler to use if :attr:`text` is a byte :class:`bytes`
        and contains some undecodable characters.
    :rtype: :class:`list` of :class:`str` strings
    :returns: list of lines that have been text wrapped and indented.

    :func:`textwrap.wrap` from the |stdlib|_ has two drawbacks that this
    attempts to fix:

    1. It does not handle :term:`textual width`.  It only operates on bytes or
       characters which are both inadequate (due to multi-byte and double
       width characters).
    2. It malforms lists and blocks.
    '''
    # Tested with:
    # yum info robodoc gpicview php-pear-Net-Socket wmctrl ustr moreutils
    #          mediawiki-HNP ocspd insight yum mousepad
    # ...at 120, 80 and 40 chars.
    # Also, notable among lots of others, searching for "\n  ":
    #   exim-clamav, jpackage-utils, tcldom, synaptics, "quake3",
    #   perl-Class-Container, ez-ipupdate, perl-Net-XMPP, "kipi-plugins",
    #   perl-Apache-DBI, netcdf, python-configobj, "translate-toolkit", alpine,
    #   "udunits", "conntrack-tools"
    #
    # Note that, we "fail" on:
    #   alsa-plugins-jack, setools*, dblatex, uisp, "perl-Getopt-GUI-Long",
    #   suitesparse, "synce-serial", writer2latex, xenwatch, ltsp-utils

    def _indent_at_beg(line):
        '''Return the indent to use for this and (possibly) subsequent lines

        :arg line: :class:`str` line of text to process
        :rtype: tuple
        :returns: tuple of count of whitespace before getting to the start of
            this line followed by a count to the following indent if this
            block of text is an entry in a list.
        '''
        # Find the first non-whitespace character
        try:
            char = line.strip()[0]
        except IndexError:
            # All whitespace
            return 0, 0
        else:
            count = line.find(char)

        # if we have a bullet character, check for list
        if char not in '-*.o\u2022\u2023\u2218':
            # No bullet; not a list
            return count, 0

        # List: Keep searching until we hit the innermost list
        nxt = _indent_at_beg(line[count+1:])
        nxt = nxt[1] or nxt[0]
        if nxt:
            return count, count + 1 + nxt
        return count, 0

    initial_indent = to_unicode(initial_indent, encoding=encoding,
            errors=errors)
    subsequent_indent = to_unicode(subsequent_indent, encoding=encoding,
            errors=errors)
    subsequent_indent_width = textual_width(subsequent_indent)

    text = to_unicode(text, encoding=encoding, errors=errors).rstrip('\n')
    lines = text.expandtabs().split('\n')

    ret = []
    indent = initial_indent
    wrap_last = False
    cur_sab = 0
    cur_spc_indent = 0
    for line in lines:
        line = line.rstrip(' ')
        (last_sab, last_spc_indent) = (cur_sab, cur_spc_indent)
        (cur_sab, cur_spc_indent) = _indent_at_beg(line)
        force_nl = False # We want to stop wrapping under "certain" conditions:
        if wrap_last and cur_spc_indent:      # if line starts a list or
            force_nl = True
        if wrap_last and cur_sab == len(line):# is empty line
            force_nl = True
        if wrap_last and not last_spc_indent: # if we don't continue a list
            if cur_sab >= 4 and cur_sab != last_sab: # and is "block indented"
                force_nl = True
        if force_nl:
            ret.append(indent.rstrip(' '))
            indent = subsequent_indent
            wrap_last = False
        if cur_sab == len(line): # empty line, remove spaces to make it easier.
            line = ''
        if wrap_last:
            line = line.lstrip(' ')
            cur_spc_indent = last_spc_indent

        if _textual_width_le(width, indent, line):
            wrap_last = False
            ret.append(indent + line)
            indent = subsequent_indent
            continue

        wrap_last = True
        words = line.split(' ')
        line = indent
        spcs = cur_spc_indent
        if not spcs and cur_sab >= 4:
            spcs = cur_sab
        for word in words:
            if (not _textual_width_le(width, line, word) and
                textual_width(line) > subsequent_indent_width):
                ret.append(line.rstrip(' '))
                line = subsequent_indent + ' ' * spcs
            line += word
            line += ' '
        indent = line.rstrip(' ') + ' '
    if wrap_last:
        ret.append(indent.rstrip(' '))

    return ret

def fill(text, *args, **kwargs):
    '''Works like we want :func:`textwrap.fill` to work

    :arg text: :class:`str` string or byte :class:`bytes` to process
    :returns: :class:`str` string with each line separated by a newline

    .. seealso::

        :func:`kitchen.text.display.wrap`
            for other parameters that you can give this command.

    This function is a light wrapper around :func:`kitchen.text.display.wrap`.
    Where that function returns a :class:`list` of lines, this function
    returns one string with each line separated by a newline.
    '''
    return '\n'.join(wrap(text, *args, **kwargs))

#
# Byte strings
#

def byte_string_textual_width_fill(msg, fill, chop=None, left=True, prefix='',
        suffix='', encoding='utf-8', errors='replace'):
    '''Expand a byte :class:`bytes` to a specified :term:`textual width` or chop
    to same

    :arg msg: byte :class:`bytes` encoded in :term:`UTF-8` that we want formatted
    :arg fill: pad :attr:`msg` until the :term:`textual width` is this long
    :kwarg chop: before doing anything else, chop the string to this length.
        Default: Don't chop the string at all
    :kwarg left: If :data:`True` (default) left justify the string and put the
        padding on the right.  If :data:`False`, pad on the left side.
    :kwarg prefix: Attach this byte :class:`bytes` before the field we're
        filling
    :kwarg suffix: Append this byte :class:`bytes` to the end of the field we're
        filling
    :rtype: byte :class:`bytes`
    :returns: :attr:`msg` formatted to fill the specified :term:`textual
        width`.  If no :attr:`chop` is specified, the string could exceed the
        fill length when completed.  If :attr:`prefix` or :attr:`suffix` are
        printable characters, the string could be longer than fill width.

    .. note::

        :attr:`prefix` and :attr:`suffix` should be used for "invisible"
        characters like highlighting, color changing escape codes, etc.  The
        fill characters are appended outside of any :attr:`prefix` or
        :attr:`suffix` elements.  This allows you to only highlight
        :attr:`msg` inside of the field you're filling.

    .. seealso::

        :func:`~kitchen.text.display.textual_width_fill`
            For example usage.  This function has only two differences.

            1. it takes byte :class:`bytes` for :attr:`prefix` and
               :attr:`suffix` so you can pass in arbitrary sequences of
               bytes, not just unicode characters.
            2. it returns a byte :class:`bytes` instead of a :class:`str`
               string.
    '''
    prefix = to_bytes(prefix, encoding=encoding, errors=errors)
    suffix = to_bytes(suffix, encoding=encoding, errors=errors)

    if chop is not None:
        msg = textual_width_chop(msg, chop, encoding=encoding, errors=errors)
    width = textual_width(msg)
    msg = to_bytes(msg)

    if width >= fill:
        if prefix or suffix:
            msg = b''.join([prefix, msg, suffix])
    else:
        extra = b' ' * (fill - width)
        if left:
            msg = b''.join([prefix, msg, suffix, extra])
        else:
            msg = b''.join([extra, prefix, msg, suffix])

    return msg

__all__ = ('byte_string_textual_width_fill', 'fill', 'textual_width',
        'textual_width_chop', 'textual_width_fill', 'wrap')



# :W0401, W0611, W0614: Rather than have two versions of subprocess, we import
#   the python2.7 version here as well
#pylint:disable-msg=W0401,W0611,W0614
import kitchen.pycompat27.subprocess as __s
from kitchen.pycompat27.subprocess import *
from kitchen.pycompat27.subprocess import __all__

import warnings

warnings.warn('In python3, kitchen.pycompat24.subprocess is deprecated.'
        '  If your code doesn\'t have to remain compatible with python less'
        ' than 2.4 use subprocess from the python3 stdlib',
        PendingDeprecationWarning, stacklevel=2)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2012 Red Hat, Inc
#
# This file is part of kitchen
#
# kitchen is free software; you can redistribute it and/or modify it under the
# terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 2.1 of the License, or (at your option)
# any later version.
#
# kitchen is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
# more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with kitchen; if not, see <http://www.gnu.org/licenses/>
#
# Authors:
#   Toshio Kuratomi <toshio@fedoraproject.org>

'''
In python-2.4, a builtin :class:`set` type was added to python.  This module
provides a function to emulate that on python-2.3 by using the :mod:`sets`
module.

:func:`set`
    Create a set.  If running on python 2.4+ this is the :class:`set`
    constructor.  If using python-2.3, it's :class:`sets.Set`.

:func:`frozenset`
    Create a frozenset.  If running on python2.4+ this is the
    :class:`frozenset` constructor.  If using python-2.3, it's
    :class:`sets.ImmutableSet`.

.. versionchanged:: 0.2.0 API: kitchen.pycompat24 1.0.0
    Added set and frozenset
'''

# All versions of python3 have set and frozenset.  This module just remains
# for compatibility
import warnings

warnings.warn('In python3, kitchen.pycompat24.sets is deprecated.'
        '  If your code doesn\'t need to maintain compatibility with'
        ' python less than 2.4, there is no reason to use anything in this'
        ' module.', PendingDeprecationWarning, stacklevel=2)

set = set
frozenset = frozenset
add_builtin_set = lambda: None

__all__ = ('add_builtin_set', 'set', 'frozenset')



#! /usr/bin/env python

"""RFC 3548: Base16, Base32, Base64 Data Encodings"""

# Modified 04-Oct-1995 by Jack Jansen to use binascii module
# Modified 30-Dec-2003 by Barry Warsaw to add full RFC 3548 support

import re
import struct
import binascii


__all__ = [
    # Legacy interface exports traditional RFC 1521 Base64 encodings
    'encode', 'decode', 'encodestring', 'decodestring',
    # Generalized interface for other encodings
    'b64encode', 'b64decode', 'b32encode', 'b32decode',
    'b16encode', 'b16decode',
    # Standard Base64 encoding
    'standard_b64encode', 'standard_b64decode',
    # Some common Base64 alternatives.  As referenced by RFC 3458, see thread
    # starting at:
    #
    # http://zgp.org/pipermail/p2p-hackers/2001-September/000316.html
    'urlsafe_b64encode', 'urlsafe_b64decode',
    ]

_translation = [chr(_x) for _x in range(256)]
EMPTYSTRING = ''


def _translate(s, altchars):
    translation = _translation[:]
    for k, v in altchars.items():
        translation[ord(k)] = v
    return s.translate(''.join(translation))



# Base64 encoding/decoding uses binascii

def b64encode(s, altchars=None):
    """Encode a string using Base64.

    s is the string to encode.  Optional altchars must be a string of at least
    length 2 (additional characters are ignored) which specifies an
    alternative alphabet for the '+' and '/' characters.  This allows an
    application to e.g. generate url or filesystem safe Base64 strings.

    The encoded string is returned.
    """
    # Strip off the trailing newline
    encoded = binascii.b2a_base64(s)[:-1]
    if altchars is not None:
        return _translate(encoded, {'+': altchars[0], '/': altchars[1]})
    return encoded


def b64decode(s, altchars=None):
    """Decode a Base64 encoded string.

    s is the string to decode.  Optional altchars must be a string of at least
    length 2 (additional characters are ignored) which specifies the
    alternative alphabet used instead of the '+' and '/' characters.

    The decoded string is returned.  A TypeError is raised if s were
    incorrectly padded or if there are non-alphabet characters present in the
    string.
    """
    if altchars is not None:
        s = _translate(s, {altchars[0]: '+', altchars[1]: '/'})
    try:
        return binascii.a2b_base64(s)
    except binascii.Error, msg:
        # Transform this exception for consistency
        raise TypeError(msg)


def standard_b64encode(s):
    """Encode a string using the standard Base64 alphabet.

    s is the string to encode.  The encoded string is returned.
    """
    return b64encode(s)

def standard_b64decode(s):
    """Decode a string encoded with the standard Base64 alphabet.

    s is the string to decode.  The decoded string is returned.  A TypeError
    is raised if the string is incorrectly padded or if there are non-alphabet
    characters present in the string.
    """
    return b64decode(s)

def urlsafe_b64encode(s):
    """Encode a string using a url-safe Base64 alphabet.

    s is the string to encode.  The encoded string is returned.  The alphabet
    uses '-' instead of '+' and '_' instead of '/'.
    """
    return b64encode(s, '-_')

def urlsafe_b64decode(s):
    """Decode a string encoded with the standard Base64 alphabet.

    s is the string to decode.  The decoded string is returned.  A TypeError
    is raised if the string is incorrectly padded or if there are non-alphabet
    characters present in the string.

    The alphabet uses '-' instead of '+' and '_' instead of '/'.
    """
    return b64decode(s, '-_')



# Base32 encoding/decoding must be done in Python
_b32alphabet = {
    0: 'A',  9: 'J', 18: 'S', 27: '3',
    1: 'B', 10: 'K', 19: 'T', 28: '4',
    2: 'C', 11: 'L', 20: 'U', 29: '5',
    3: 'D', 12: 'M', 21: 'V', 30: '6',
    4: 'E', 13: 'N', 22: 'W', 31: '7',
    5: 'F', 14: 'O', 23: 'X',
    6: 'G', 15: 'P', 24: 'Y',
    7: 'H', 16: 'Q', 25: 'Z',
    8: 'I', 17: 'R', 26: '2',
    }

_b32tab = _b32alphabet.items()
_b32tab.sort()
_b32tab = [v for k, v in _b32tab]
_b32rev = dict([(v, long(k)) for k, v in _b32alphabet.items()])


def b32encode(s):
    """Encode a string using Base32.

    s is the string to encode.  The encoded string is returned.
    """
    parts = []
    quanta, leftover = divmod(len(s), 5)
    # Pad the last quantum with zero bits if necessary
    if leftover:
        s += ('\0' * (5 - leftover))
        quanta += 1
    for i in range(quanta):
        # c1 and c2 are 16 bits wide, c3 is 8 bits wide.  The intent of this
        # code is to process the 40 bits in units of 5 bits.  So we take the 1
        # leftover bit of c1 and tack it onto c2.  Then we take the 2 leftover
        # bits of c2 and tack them onto c3.  The shifts and masks are intended
        # to give us values of exactly 5 bits in width.
        c1, c2, c3 = struct.unpack('!HHB', s[i*5:(i+1)*5])
        c2 += (c1 & 1) << 16 # 17 bits wide
        c3 += (c2 & 3) << 8  # 10 bits wide
        parts.extend([_b32tab[c1 >> 11],         # bits 1 - 5
                      _b32tab[(c1 >> 6) & 0x1f], # bits 6 - 10
                      _b32tab[(c1 >> 1) & 0x1f], # bits 11 - 15
                      _b32tab[c2 >> 12],         # bits 16 - 20 (1 - 5)
                      _b32tab[(c2 >> 7) & 0x1f], # bits 21 - 25 (6 - 10)
                      _b32tab[(c2 >> 2) & 0x1f], # bits 26 - 30 (11 - 15)
                      _b32tab[c3 >> 5],          # bits 31 - 35 (1 - 5)
                      _b32tab[c3 & 0x1f],        # bits 36 - 40 (1 - 5)
                      ])
    encoded = EMPTYSTRING.join(parts)
    # Adjust for any leftover partial quanta
    if leftover == 1:
        return encoded[:-6] + '======'
    elif leftover == 2:
        return encoded[:-4] + '===='
    elif leftover == 3:
        return encoded[:-3] + '==='
    elif leftover == 4:
        return encoded[:-1] + '='
    return encoded


def b32decode(s, casefold=False, map01=None):
    """Decode a Base32 encoded string.

    s is the string to decode.  Optional casefold is a flag specifying whether
    a lowercase alphabet is acceptable as input.  For security purposes, the
    default is False.

    RFC 3548 allows for optional mapping of the digit 0 (zero) to the letter O
    (oh), and for optional mapping of the digit 1 (one) to either the letter I
    (eye) or letter L (el).  The optional argument map01 when not None,
    specifies which letter the digit 1 should be mapped to (when map01 is not
    None, the digit 0 is always mapped to the letter O).  For security
    purposes the default is None, so that 0 and 1 are not allowed in the
    input.

    The decoded string is returned.  A TypeError is raised if s were
    incorrectly padded or if there are non-alphabet characters present in the
    string.
    """
    quanta, leftover = divmod(len(s), 8)
    if leftover:
        raise TypeError('Incorrect padding')
    # Handle section 2.4 zero and one mapping.  The flag map01 will be either
    # False, or the character to map the digit 1 (one) to.  It should be
    # either L (el) or I (eye).
    if map01:
        s = _translate(s, {'0': 'O', '1': map01})
    if casefold:
        s = s.upper()
    # Strip off pad characters from the right.  We need to count the pad
    # characters because this will tell us how many null bytes to remove from
    # the end of the decoded string.
    padchars = 0
    mo = re.search('(?P<pad>[=]*)$', s)
    if mo:
        padchars = len(mo.group('pad'))
        if padchars > 0:
            s = s[:-padchars]
    # Now decode the full quanta
    parts = []
    acc = 0
    shift = 35
    for c in s:
        val = _b32rev.get(c)
        if val is None:
            raise TypeError('Non-base32 digit found')
        acc += _b32rev[c] << shift
        shift -= 5
        if shift < 0:
            parts.append(binascii.unhexlify('%010x' % acc))
            acc = 0
            shift = 35
    # Process the last, partial quanta
    last = binascii.unhexlify('%010x' % acc)
    if padchars == 0:
        last = ''                       # No characters
    elif padchars == 1:
        last = last[:-1]
    elif padchars == 3:
        last = last[:-2]
    elif padchars == 4:
        last = last[:-3]
    elif padchars == 6:
        last = last[:-4]
    else:
        raise TypeError('Incorrect padding')
    parts.append(last)
    return EMPTYSTRING.join(parts)



# RFC 3548, Base 16 Alphabet specifies uppercase, but hexlify() returns
# lowercase.  The RFC also recommends against accepting input case
# insensitively.
def b16encode(s):
    """Encode a string using Base16.

    s is the string to encode.  The encoded string is returned.
    """
    return binascii.hexlify(s).upper()


def b16decode(s, casefold=False):
    """Decode a Base16 encoded string.

    s is the string to decode.  Optional casefold is a flag specifying whether
    a lowercase alphabet is acceptable as input.  For security purposes, the
    default is False.

    The decoded string is returned.  A TypeError is raised if s were
    incorrectly padded or if there are non-alphabet characters present in the
    string.
    """
    if casefold:
        s = s.upper()
    if re.search('[^0-9A-F]', s):
        raise TypeError('Non-base16 digit found')
    return binascii.unhexlify(s)



# Legacy interface.  This code could be cleaned up since I don't believe
# binascii has any line length limitations.  It just doesn't seem worth it
# though.

MAXLINESIZE = 76 # Excluding the CRLF
MAXBINSIZE = (MAXLINESIZE//4)*3

def encode(input, output):
    """Encode a file."""
    while True:
        s = input.read(MAXBINSIZE)
        if not s:
            break
        while len(s) < MAXBINSIZE:
            ns = input.read(MAXBINSIZE-len(s))
            if not ns:
                break
            s += ns
        line = binascii.b2a_base64(s)
        output.write(line)


def decode(input, output):
    """Decode a file."""
    while True:
        line = input.readline()
        if not line:
            break
        s = binascii.a2b_base64(line)
        output.write(s)


def encodestring(s):
    """Encode a string into multiple lines of base-64 data."""
    pieces = []
    for i in range(0, len(s), MAXBINSIZE):
        chunk = s[i: i + MAXBINSIZE]
        pieces.append(binascii.b2a_base64(chunk))
    return "".join(pieces)


def decodestring(s):
    """Decode a string."""
    return binascii.a2b_base64(s)



# Useable as a script...
def test():
    """Small test program"""
    import sys, getopt
    try:
        opts, args = getopt.getopt(sys.argv[1:], 'deut')
    except getopt.error, msg:
        sys.stdout = sys.stderr
        print msg
        print """usage: %s [-d|-e|-u|-t] [file|-]
        -d, -u: decode
        -e: encode (default)
        -t: encode and decode string 'Aladdin:open sesame'"""%sys.argv[0]
        sys.exit(2)
    func = encode
    for o, a in opts:
        if o == '-e': func = encode
        if o == '-d': func = decode
        if o == '-u': func = decode
        if o == '-t': test1(); return
    if args and args[0] != '-':
        fh = open(args[0], 'rb')
        try:
            func(fh, sys.stdout)
        finally:
            fh.close()
    else:
        func(sys.stdin, sys.stdout)


def test1():
    s0 = "Aladdin:open sesame"
    s1 = encodestring(s0)
    s2 = decodestring(s1)
    print s0, repr(s1), s2


if __name__ == '__main__':
    test()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2011-2012 Red Hat, Inc
#
# kitchen is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# kitchen is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with kitchen; if not, see <http://www.gnu.org/licenses/>
#
# Authors:
#   Toshio Kuratomi <toshio@fedoraproject.org>
#
'''
Kitchen

Aggregate of a bunch of unrelated but helpful python modules.
'''

# Pylint disabled messages:
# :C0103: We need gettext aliases for both unicode strings and byte strings.
#   The byte string one (b_) triggers this warning.
from kitchen import i18n
from kitchen import versioning

(_, N_) = i18n.easy_gettext_setup('kitchen.core')
#pylint: disable-msg=C0103
(b_, bN_) = i18n.easy_gettext_setup('kitchen.core', use_unicode=False)
#pylint: enable-msg=C0103

__version_info__ = ((1, 2, 6),)
__version__ = versioning.version_tuple_to_string(__version_info__)

__all__ = ('exceptions', 'release',)



# -*- coding: utf-8 -*-
# Copyright (c) 2012 Red Hat, Inc
# Copyright (c) 2010 Seth Vidal
#
# kitchen is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# kitchen is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with kitchen; if not, see <http://www.gnu.org/licenses/>
#
# Authors:
#   James Antill
#   Toshio Kuratomi <toshio@fedoraproject.org>
#   Seth Vidal
#
# Portions of this code taken from yum/misc.py and yum/i18n.py
'''
---------------------------------------------
Miscellaneous functions for manipulating text
---------------------------------------------

Collection of text functions that don't fit in another category.

.. versionchanged:: kitchen 1.2.0, API: kitchen.text 2.2.0
    Added :func:`~kitchen.text.misc.isbasestring`,
    :func:`~kitchen.text.misc.isbytestring`, and
    :func:`~kitchen.text.misc.isunicodestring` to help tell which string type
    is which on python2 and python3
'''
import html.entities
import itertools
import re

try:
    import chardet
except ImportError:
    chardet = None

from kitchen.text.exceptions import ControlCharError

# Define a threshold for chardet confidence.  If we fall below this we decode
# byte strings we're guessing about as latin1
_CHARDET_THRESHHOLD = 0.6

# ASCII control codes (the c0 codes) that are illegal in xml 1.0
# Also unicode control codes (the C1 codes): also illegal in xml
_CONTROL_CODES = frozenset(itertools.chain(range(0, 8), (11, 12), range(14, 32), range(128, 160)))
_CONTROL_CHARS = frozenset(map(chr, _CONTROL_CODES))
_IGNORE_TABLE = dict(zip(_CONTROL_CODES, [None] * len(_CONTROL_CODES)))
_REPLACE_TABLE = dict(zip(_CONTROL_CODES, ['?'] * len(_CONTROL_CODES)))

# _ENTITY_RE
_ENTITY_RE = re.compile(r'(?s)<[^>]*>|&#?\w+;')

def isbasestring(obj):
    '''Determine if obj is a byte :class:`bytes` or :class:`str` string

    In python2 this is eqiuvalent to isinstance(obj, basestring).  In python3
    it checks whether the object is an instance of str, bytes, or bytearray.
    This is an aid to porting code that needed to test whether an object was
    derived from basestring in python2 (commonly used in unicode-bytes
    conversion functions)

    :arg obj: Object to test
    :returns: True if the object is a :class:`basestring`.  Otherwise False.

    .. versionadded:: Kitchen: 1.2.0, API kitchen.text 2.2.0
    '''
    if isinstance(obj, (str, bytes, bytearray)):
        return True
    return False

def isbytestring(obj):
    '''Determine if obj is a byte :class:`bytes`

    In python2 this is equivalent to isinstance(obj, str).  In python3 it
    checks whether the object is an instance of bytes or bytearray.

    :arg obj: Object to test
    :returns: True if the object is a byte :class:`bytes`.  Otherwise, False.

    .. versionadded:: Kitchen: 1.2.0, API kitchen.text 2.2.0
    '''
    if isinstance(obj, (bytes, bytearray)):
        return True
    return False

def isunicodestring(obj):
    '''Determine if obj is a :class:`str` string

    In python2 this is equivalent to isinstance(obj, unicode).  In python3 it
    checks whether the object is an instance of :class:`bytes`.

    :arg obj: Object to test
    :returns: True if the object is a :class:`str` string.  Otherwise, False.

    .. versionadded:: Kitchen: 1.2.0, API kitchen.text 2.2.0
    '''
    if isinstance(obj, str):
        return True
    return False

def guess_encoding(byte_string, disable_chardet=False):
    '''Try to guess the encoding of a byte :class:`bytes`

    :arg byte_string: byte :class:`bytes` to guess the encoding of
    :kwarg disable_chardet: If this is True, we never attempt to use
        :mod:`chardet` to guess the encoding.  This is useful if you need to
        have reproducibility whether :mod:`chardet` is installed or not.
        Default: :data:`False`.
    :raises TypeError: if :attr:`byte_string` is not a byte :class:`bytes` type
    :returns: string containing a guess at the encoding of
        :attr:`byte_string`.  This is appropriate to pass as the encoding
        argument when encoding and decoding unicode strings.

    We start by attempting to decode the byte :class:`bytes` as :term:`UTF-8`.
    If this succeeds we tell the world it's :term:`UTF-8` text.  If it doesn't
    and :mod:`chardet` is installed on the system and :attr:`disable_chardet`
    is False this function will use it to try detecting the encoding of
    :attr:`byte_string`.  If it is not installed or :mod:`chardet` cannot
    determine the encoding with a high enough confidence then we rather
    arbitrarily claim that it is ``latin-1``.  Since ``latin-1`` will encode
    to every byte, decoding from ``latin-1`` to :class:`str` will not
    cause :exc:`UnicodeErrors` although the output might be mangled.
    '''
    if not isbytestring(byte_string):
        raise TypeError('byte_string must be a byte string (bytes, bytearray)')
    input_encoding = 'utf-8'
    try:
        str(byte_string, input_encoding, 'strict')
    except UnicodeDecodeError:
        input_encoding = None

    if not input_encoding and chardet and not disable_chardet:
        detection_info = chardet.detect(byte_string)
        if detection_info['confidence'] >= _CHARDET_THRESHHOLD:
            input_encoding = detection_info['encoding']

    if not input_encoding:
        input_encoding = 'latin-1'

    return input_encoding

def str_eq(str1, str2, encoding='utf-8', errors='replace'):
    '''Compare two strings, converting to byte :class:`bytes` if one is
    :class:`str`

    :arg str1: First string to compare
    :arg str2: Second string to compare
    :kwarg encoding: If we need to convert one string into a byte :class:`bytes`
        to compare, the encoding to use.  Default is :term:`utf-8`.
    :kwarg errors: What to do if we encounter errors when encoding the string.
        See the :func:`kitchen.text.converters.to_bytes` documentation for
        possible values.  The default is ``replace``.

    This function prevents :exc:`UnicodeError` (python-2.4 or less) and
    :exc:`UnicodeWarning` (python 2.5 and higher) when we compare
    a :class:`str` string to a byte :class:`bytes`.  The errors normally
    arise because the conversion is done to :term:`ASCII`.  This function
    lets you convert to :term:`utf-8` or another encoding instead.

    .. note::

        When we need to convert one of the strings from :class:`str` in
        order to compare them we convert the :class:`str` string into
        a byte :class:`bytes`.  That means that strings can compare differently
        if you use different encodings for each.

    Note that ``str1 == str2`` is faster than this function if you can accept
    the following limitations:

    * Limited to python-2.5+ (otherwise a :exc:`UnicodeDecodeError` may be
      thrown)
    * Will generate a :exc:`UnicodeWarning` if non-:term:`ASCII` byte
      :class:`bytes` is compared to :class:`str` string.
    '''
    try:
        return (not str1 < str2) and (not str1 > str2)
    except TypeError:
        pass

    if isunicodestring(str1):
        str1 = str1.encode(encoding, errors)
    else:
        str2 = str2.encode(encoding, errors)
    if str1 == str2:
        return True

    return False

def process_control_chars(string, strategy='replace'):
    '''Look for and transform :term:`control characters` in a string

    :arg string: string to search for and transform :term:`control characters`
        within
    :kwarg strategy: XML does not allow :term:`ASCII` :term:`control
        characters`.  When we encounter those we need to know what to do.
        Valid options are:

        :replace: (default) Replace the :term:`control characters`
            with ``"?"``
        :ignore: Remove the characters altogether from the output
        :strict: Raise a :exc:`~kitchen.text.exceptions.ControlCharError` when
            we encounter a control character
    :raises TypeError: if :attr:`string` is not a unicode string.
    :raises ValueError: if the strategy is not one of replace, ignore, or
        strict.
    :raises kitchen.text.exceptions.ControlCharError: if the strategy is
        ``strict`` and a :term:`control character` is present in the
        :attr:`string`
    :returns: :class:`str` string with no :term:`control characters` in
        it.

    .. versionchanged:: kitchen 1.2.0, API: kitchen.text 2.2.0
        Strip out the C1 control characters in addition to the C0 control
        characters.
    '''
    if not isunicodestring(string):
        raise TypeError('process_control_char must have a unicode type'
                ' (str) as the first argument.')
    if strategy not in ('replace', 'ignore', 'strict'):
        raise ValueError('The strategy argument to process_control_chars'
                ' must be one of ignore, replace, or strict')

    # Most strings don't have control chars and translating carries
    # a higher cost than testing whether the chars are in the string
    # So only translate if necessary
    if not _CONTROL_CHARS.isdisjoint(string):
        if strategy == 'replace':
            control_table = _REPLACE_TABLE
        elif strategy == 'ignore':
            control_table = _IGNORE_TABLE
        else:
            # strategy can only equal 'strict'
            raise ControlCharError('ASCII control code present in string'
                    ' input')
        string = string.translate(control_table)

    return string

# Originally written by Fredrik Lundh (January 15, 2003) and placed in the
# public domain::
#
#   Unless otherwise noted, source code can be be used freely. Examples, test
#   scripts and other short code fragments can be considered as being in the
#   public domain.
#
# http://effbot.org/zone/re-sub.htm#unescape-html
# http://effbot.org/zone/copyright.htm
#
def html_entities_unescape(string):
    '''Substitute unicode characters for HTML entities

    :arg string: :class:`str` string to substitute out html entities
    :raises TypeError: if something other than a :class:`str` string is
        given
    :rtype: :class:`str` string
    :returns: The plain text without html entities
    '''
    def fixup(match):
        string = match.group(0)
        if string[:1] == "<":
            return "" # ignore tags
        if string[:2] == "&#":
            try:
                if string[:3] == "&#x":
                    return chr(int(string[3:-1], 16))
                else:
                    return chr(int(string[2:-1]))
            except ValueError:
                # If the value is outside the unicode codepoint range, leave
                # it in the output as is
                pass
        elif string[:1] == "&":
            entity = html.entities.entitydefs.get(string[1:-1])
            if entity:
                if entity[:2] == "&#":
                    try:
                        return chr(int(entity[2:-1]))
                    except ValueError:
                        # If the value is outside the unicode codepoint range,
                        # leave it in the output as is
                        pass
                else:
                    return entity
        return string # leave as is

    if not isunicodestring(string):
        raise TypeError('html_entities_unescape must have a unicode type (str)'
                ' for its first argument')
    return re.sub(_ENTITY_RE, fixup, string)

def byte_string_valid_xml(byte_string, encoding='utf-8'):
    '''Check that a byte :class:`bytes` would be valid in xml

    :arg byte_string: Byte :class:`bytes` to check
    :arg encoding: Encoding of the xml file.  Default: :term:`UTF-8`
    :returns: :data:`True` if the string is valid.  :data:`False` if it would
        be invalid in the xml file

    In some cases you'll have a whole bunch of byte strings and rather than
    transforming them to :class:`str` and back to byte :class:`bytes` for
    output to xml, you will just want to make sure they work with the xml file
    you're constructing.  This function will help you do that.  Example::

        ARRAY_OF_MOSTLY_UTF8_STRINGS = [...]
        processed_array = []
        for string in ARRAY_OF_MOSTLY_UTF8_STRINGS:
            if byte_string_valid_xml(string, 'utf-8'):
                processed_array.append(string)
            else:
                processed_array.append(guess_bytes_to_xml(string, encoding='utf-8'))
        output_xml(processed_array)
    '''
    if not isbytestring(byte_string):
        # Not a byte string
        return False

    try:
        u_string = str(byte_string, encoding)
    except UnicodeError:
        # Not encoded with the xml file's encoding
        return False

    data = frozenset(u_string)
    if data.intersection(_CONTROL_CHARS):
        # Contains control codes
        return False

    # The byte string is compatible with this xml file
    return True

def byte_string_valid_encoding(byte_string, encoding='utf-8'):
    '''Detect if a byte :class:`bytes` is valid in a specific encoding

    :arg byte_string: Byte :class:`bytes` to test for bytes not valid in this
        encoding
    :kwarg encoding: encoding to test against.  Defaults to :term:`UTF-8`.
    :returns: :data:`True` if there are no invalid :term:`UTF-8` characters.
        :data:`False` if an invalid character is detected.

    .. note::

        This function checks whether the byte :class:`bytes` is valid in the
        specified encoding.  It **does not** detect whether the byte
        :class:`bytes` actually was encoded in that encoding.  If you want that
        sort of functionality, you probably want to use
        :func:`~kitchen.text.misc.guess_encoding` instead.
    '''
    try:
        str(byte_string, encoding)
    except UnicodeError:
        # Not encoded with the xml file's encoding
        return False

    # byte string is valid in this encoding
    return True

__all__ = ('byte_string_valid_encoding', 'byte_string_valid_xml',
        'guess_encoding', 'html_entities_unescape', 'isbasestring',
        'isbytestring', 'isunicodestring', 'process_control_chars', 'str_eq')



'''
The :mod:`kitchen.pycompat27` module contains implementations of functionality
introduced in python-2.7 for use on earlier versions of python.

.. versionchanged:: 0.2.3
    Made mswindows, MAXFD, and list2cmdline available from the module
'''
import warnings

warnings.warn('In python3, kitchen.pycompat27 is deprecated because all'
        ' functionality in this module is found in the python3 stdlib.'
        '  If your code doesn\'t need to maintain compatibility with'
        ' python less than 2.7, use subprocess from the python3 stdlib.',
        PendingDeprecationWarning, stacklevel=2)

from kitchen.versioning import version_tuple_to_string

__version_info__ = ((1, 1, 0),)
__version__ = version_tuple_to_string(__version_info__)

__all__ = ('subprocess',)



# -*- coding: utf-8 -*-
from nose import tools

import os
import types
import warnings
from kitchen.pycompat24.sets import add_builtin_set
add_builtin_set()

def logit(msg):
    log = open('/var/tmp/test.log', 'a')
    log.write('%s\n' % msg)
    log.close()

class NoAll(RuntimeError):
    pass

class FailedImport(RuntimeError):
    pass

class Test__all__(object):
    '''Test that every function in __all__ exists and that no public methods
    are missing from __all__
    '''
    known_private = set([('kitchen', 'collections', 'version_tuple_to_string'),
        ('kitchen.collections', 'strictdict', 'defaultdict'),
        ('kitchen', 'i18n', 'version_tuple_to_string'),
        ('kitchen', 'i18n', 'to_bytes'),
        ('kitchen', 'i18n', 'to_unicode'),
        ('kitchen', 'i18n', 'ENOENT'),
        ('kitchen', 'i18n', 'byte_string_valid_encoding'),
        ('kitchen', 'i18n', 'isbasestring'),
        ('kitchen', 'i18n', 'partial'),
        ('kitchen', 'iterutils', 'isbasestring'),
        ('kitchen', 'iterutils', 'version_tuple_to_string'),
        ('kitchen', 'pycompat24', 'version_tuple_to_string'),
        ('kitchen', 'pycompat25', 'version_tuple_to_string'),
        ('kitchen.pycompat25.collections', '_defaultdict', 'b_'),
        ('kitchen', 'pycompat27', 'version_tuple_to_string'),
        ('kitchen.pycompat27', 'subprocess', 'MAXFD'),
        ('kitchen.pycompat27', 'subprocess', 'list2cmdline'),
        ('kitchen.pycompat27', 'subprocess', 'mswindows'),
        ('kitchen', 'text', 'version_tuple_to_string'),
        ('kitchen.text', 'converters', 'b_'),
        ('kitchen.text', 'converters', 'b64decode'),
        ('kitchen.text', 'converters', 'b64encode'),
        ('kitchen.text', 'converters', 'ControlCharError'),
        ('kitchen.text', 'converters', 'guess_encoding'),
        ('kitchen.text', 'converters', 'html_entities_unescape'),
        ('kitchen.text', 'converters', 'isbytestring'),
        ('kitchen.text', 'converters', 'isunicodestring'),
        ('kitchen.text', 'converters', 'process_control_chars'),
        ('kitchen.text', 'converters', 'XmlEncodeError'),
        ('kitchen.text', 'misc', 'b_'),
        ('kitchen.text', 'misc', 'chardet'),
        ('kitchen.text', 'misc', 'ControlCharError'),
        ('kitchen.text', 'display', 'b_'),
        ('kitchen.text', 'display', 'ControlCharError'),
        ('kitchen.text', 'display', 'to_bytes'),
        ('kitchen.text', 'display', 'to_unicode'),
        ('kitchen.text', 'utf8', 'b_'),
        ('kitchen.text', 'utf8', 'byte_string_textual_width_fill'),
        ('kitchen.text', 'utf8', 'byte_string_valid_encoding'),
        ('kitchen.text', 'utf8', 'fill'),
        ('kitchen.text', 'utf8', 'isunicodestring'),
        ('kitchen.text', 'utf8', 'textual_width'),
        ('kitchen.text', 'utf8', 'textual_width_chop'),
        ('kitchen.text', 'utf8', 'to_bytes'),
        ('kitchen.text', 'utf8', 'to_unicode'),
        ('kitchen.text', 'utf8', 'wrap'),
        ])
    lib_dir = os.path.join(os.path.dirname(__file__), '..', 'kitchen')

    def setUp(self):
        # Silence deprecation warnings
        warnings.simplefilter('ignore', DeprecationWarning)
    def tearDown(self):
        warnings.simplefilter('default', DeprecationWarning)

    def walk_modules(self, basedir, modpath):
        files = os.listdir(basedir)
        files.sort()
        for fn in files:
            path = os.path.join(basedir, fn)
            if os.path.isdir(path):
                pkg_init = os.path.join(path, '__init__.py')
                if os.path.exists(pkg_init):
                    yield pkg_init, modpath + fn
                    for p, m in self.walk_modules(path, modpath + fn + '.'):
                        yield p, m
                continue
            if not fn.endswith('.py') or fn == '__init__.py':
                continue
            yield path, modpath + fn[:-3]

    def check_has__all__(self, modpath):
        # This heuristic speeds up the process by removing, de facto,
        # most test modules (and avoiding the auto-executing ones).
        f = None
        try:
            try:
                f = open(modpath, 'rb')
                tools.ok_('__all__' in f.read(), '%s does not contain __all__' % modpath)
            except IOError, e:
                tools.ok_(False, '%s' % e)
        finally:
            if f:
                f.close()

    def test_has__all__(self):
        '''
        For each module, check that it has an __all__
        '''
        # Blacklisted modules and packages
        blacklist = set([])

        for path, modname in [m for m in self.walk_modules(self.lib_dir, '')
                if m[1] not in blacklist]:
            # Check that it has an __all__
            yield self.check_has__all__, path

    def check_everything_in__all__exists(self, modname, modpath):
        names = {}
        exec 'from %s import %s' % (modpath, modname) in names
        if not hasattr(names[modname], '__all__'):
            # This should have been reported by test_has__all__
            return

        interior_names = {}
        try:
            exec 'from %s.%s import *' % (modpath, modname) in interior_names
        except Exception, e:
            # Include the module name in the exception string
            tools.ok_(False, '__all__ failure in %s: %s: %s' % (
                      modname, e.__class__.__name__, e))
        if '__builtins__' in interior_names:
            del interior_names['__builtins__']
        keys = set(interior_names)
        all = set(names[modname].__all__)
        tools.ok_(keys == all)

    def test_everything_in__all__exists(self):
        '''
        For each name in module's __all__, check that it exists
        '''
        # Blacklisted modules and packages
        blacklist = set([])

        for path, modname in [m for m in self.walk_modules(self.lib_dir, '')
                if m[1] not in blacklist]:
            # From path, deduce the module name
            from_name = path[path.find('../kitchen') + 3:]
            if from_name.endswith('__init__.py'):
                # Remove __init__.py as well as the filename
                from_name = os.path.dirname(from_name)
            from_name = os.path.dirname(from_name)
            from_name = unicode(from_name, 'utf-8')
            from_name = from_name.translate({ord(u'/'): u'.'})
            from_name = from_name.encode('utf-8')
            yield self.check_everything_in__all__exists, modname.split('.')[-1], from_name


    def check__all__is_complete(self, modname, modpath):
        names = {}
        exec 'from %s import %s' % (modpath, modname) in names
        if not hasattr(names[modname], '__all__'):
            # This should have been reported by test_has__all__
            return

        mod = names[modname]
        expected_public = [k for k in mod.__dict__ if (modpath, modname, k)
                not in self.known_private and not k.startswith("_") and not
                isinstance(mod.__dict__[k], types.ModuleType)]

        all = set(mod.__all__)
        public = set(expected_public)
        tools.ok_(all.issuperset(public), 'These public names are not in %s.__all__: %s'
                % (modname, ', '.join(public.difference(all))))

    def test__all__is_complete(self):
        '''
        For each module, check that every public name is in __all__
        '''
        # Blacklisted modules and packages
        blacklist = set(['pycompat27.subprocess._subprocess',
            'pycompat24.base64._base64'])

        for path, modname in [m for m in self.walk_modules(self.lib_dir, '')
                if m[1] not in blacklist]:
            # From path, deduce the module name
            from_name = path[path.find('../kitchen') + 3:]
            if from_name.endswith('__init__.py'):
                # Remove __init__.py as well as the filename
                from_name = os.path.dirname(from_name)
            from_name = os.path.dirname(from_name)
            from_name = unicode(from_name, 'utf-8')
            from_name = from_name.translate({ord(u'/'): u'.'})
            from_name = from_name.encode('utf-8')
            yield self.check__all__is_complete, modname.split('.')[-1], from_name



# -*- coding: utf-8 -*-
#
# Copyright (c) 2012 Red Hat, Inc
#
# kitchen is free software; you can redistribute it and/or modify it under the
# terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 2.1 of the License, or (at your option)
# any later version.
#.
# kitchen is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
# more details.
#.
# You should have received a copy of the GNU Lesser General Public License
# along with kitchen; if not, see <http://www.gnu.org/licenses/>
#
# Authors:
#   Toshio Kuratomi <toshio@fedoraproject.org>
#   Luke Macken <lmacken@redhat.com>
#
# Portions of code taken from python-fedora fedora/iterutils.py
'''
Functions to manipulate iterables

.. versionadded:: Kitchen: 0.2.1a1

.. moduleauthor:: Toshio Kuratomi <toshio@fedoraproject.org>
.. moduleauthor:: Luke Macken <lmacken@redhat.com>
'''

from kitchen.versioning import version_tuple_to_string

__version_info__ = ((0, 0, 1),)
__version__ = version_tuple_to_string(__version_info__)

from kitchen.text.misc import isbasestring

def isiterable(obj, include_string=False):
    '''Check whether an object is an iterable

    :arg obj: Object to test whether it is an iterable
    :kwarg include_string: If :data:`True` and :attr:`obj` is a byte
        :class:`str` or :class:`unicode` string this function will return
        :data:`True`.  If set to :data:`False`, byte :class:`str` and
        :class:`unicode` strings will cause this function to return
        :data:`False`.  Default :data:`False`.
    :returns: :data:`True` if :attr:`obj` is iterable, otherwise
        :data:`False`.
    '''
    if include_string or not isbasestring(obj):
        try:
            iter(obj)
        except TypeError:
            return False
        else:
            return True
    return False

def iterate(obj, include_string=False):
    '''Generator that can be used to iterate over anything

    :arg obj: The object to iterate over
    :kwarg include_string: if :data:`True`, treat strings as iterables.
        Otherwise treat them as a single scalar value.  Default :data:`False`

    This function will create an iterator out of any scalar or iterable.  It
    is useful for making a value given to you an iterable before operating on it.
    Iterables have their items returned.  scalars are transformed into iterables.
    A string is treated as a scalar value unless the :attr:`include_string`
    parameter is set to :data:`True`.  Example usage::

        >>> list(iterate(None))
        [None]
        >>> list(iterate([None]))
        [None]
        >>> list(iterate([1, 2, 3]))
        [1, 2, 3]
        >>> list(iterate(set([1, 2, 3])))
        [1, 2, 3]
        >>> list(iterate(dict(a='1', b='2')))
        ['a', 'b']
        >>> list(iterate(1))
        [1]
        >>> list(iterate(iter([1, 2, 3])))
        [1, 2, 3]
        >>> list(iterate('abc'))
        ['abc']
        >>> list(iterate('abc', include_string=True))
        ['a', 'b', 'c']
    '''
    if isiterable(obj, include_string=include_string):
        for item in obj:
            yield item
    else:
        yield obj

__all__ = ('isiterable', 'iterate',)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2012 Red Hat, Inc.
# Copyright (c) 2010 Ville Skyttä
# Copyright (c) 2009 Tim Lauridsen
# Copyright (c) 2007 Marcus Kuhn
#
# kitchen is free software; you can redistribute it and/or modify it under the
# terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 2.1 of the License, or (at your option)
# any later version.
#
# kitchen is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
# more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with kitchen; if not, see <http://www.gnu.org/licenses/>
#
# Authors:
#   James Antill <james@fedoraproject.org>
#   Marcus Kuhn
#   Toshio Kuratomi <toshio@fedoraproject.org>
#   Tim Lauridsen
#   Ville Skyttä
#
# Portions of this are from yum/i18n.py
'''
-----
UTF-8
-----

Functions for operating on byte :class:`bytes` encoded as :term:`UTF-8`

.. note::

    In many cases, it is better to convert to :class:`str`, operate on the
    strings, then convert back to :term:`UTF-8`.  :class:`str` type can
    handle many of these functions itself.  For those that it doesn't
    (removing control characters from length calculations, for instance) the
    code to do so with a :class:`str` type is often simpler.

.. warning::

    All of the functions in this module are deprecated.  Most of them have
    been replaced with functions that operate on unicode values in
    :mod:`kitchen.text.display`.  :func:`kitchen.text.utf8.utf8_valid` has
    been replaced with a function in :mod:`kitchen.text.misc`.
'''
import warnings

from kitchen.text.converters import to_unicode, to_bytes
from kitchen.text.misc import byte_string_valid_encoding, isunicodestring
from kitchen.text.display import _textual_width_le, \
        byte_string_textual_width_fill, fill, textual_width, \
        textual_width_chop, wrap

#
# Deprecated functions
#

def utf8_valid(msg):
    '''**Deprecated** Detect if a string is valid :term:`utf-8`

    Use :func:`kitchen.text.misc.byte_string_valid_encoding` instead.
    '''
    warnings.warn('kitchen.text.utf8.utf8_valid is deprecated.  Use'
            ' kitchen.text.misc.byte_string_valid_encoding(msg) instead',
            DeprecationWarning, stacklevel=2)
    return byte_string_valid_encoding(msg)

def utf8_width(msg):
    '''**Deprecated** Get the :term:`textual width` of a :term:`utf-8` string

    Use :func:`kitchen.text.display.textual_width` instead.
    '''
    warnings.warn('kitchen.text.utf8.utf8_width is deprecated.  Use'
        ' kitchen.text.display.textual_width(msg) instead',
        DeprecationWarning, stacklevel=2)
    return textual_width(msg)


def utf8_width_chop(msg, chop=None):
    '''**Deprecated** Return a string chopped to a given :term:`textual width`

    Use :func:`~kitchen.text.display.textual_width_chop` and
    :func:`~kitchen.text.display.textual_width` instead::

        >>> msg = 'く ku ら ra と to み mi'
        >>> # Old way:
        >>> utf8_width_chop(msg, 5)
        (5, 'く ku')
        >>> # New way
        >>> from kitchen.text.converters import to_bytes
        >>> from kitchen.text.display import textual_width, textual_width_chop
        >>> (textual_width(msg), to_bytes(textual_width_chop(msg, 5)))
        (5, 'く ku')
    '''
    warnings.warn('kitchen.text.utf8.utf8_width_chop is deprecated.  Use'
        ' kitchen.text.display.textual_width_chop instead', DeprecationWarning,
        stacklevel=2)

    if chop == None:
        return textual_width(msg), msg

    as_bytes = not isunicodestring(msg)

    chopped_msg = textual_width_chop(msg, chop)
    if as_bytes:
        chopped_msg = to_bytes(chopped_msg)
    return textual_width(chopped_msg), chopped_msg

def utf8_width_fill(msg, fill, chop=None, left=True, prefix='', suffix=''):
    '''**Deprecated** Pad a :term:`utf-8` string to fill a specified width

    Use :func:`~kitchen.text.display.byte_string_textual_width_fill` instead
    '''
    warnings.warn('kitchen.text.utf8.utf8_width_fill is deprecated.  Use'
        ' kitchen.text.display.byte_string_textual_width_fill instead',
        DeprecationWarning, stacklevel=2)

    return byte_string_textual_width_fill(msg, fill, chop=chop, left=left,
            prefix=prefix, suffix=suffix)

def utf8_text_wrap(text, width=70, initial_indent='', subsequent_indent=''):
    '''**Deprecated** Similar to :func:`textwrap.wrap` but understands
    :term:`utf-8` data and doesn't screw up lists/blocks/etc

    Use :func:`kitchen.text.display.wrap` instead
    '''
    warnings.warn('kitchen.text.utf8.utf8_text_wrap is deprecated.  Use'
        ' kitchen.text.display.wrap instead',
        DeprecationWarning, stacklevel=2)

    as_bytes = not isunicodestring(text)

    text = to_unicode(text)
    lines = wrap(text, width=width, initial_indent=initial_indent,
            subsequent_indent=subsequent_indent)
    if as_bytes:
        lines = [to_bytes(m) for m in lines]

    return lines

def utf8_text_fill(text, *args, **kwargs):
    '''**Deprecated** Similar to :func:`textwrap.fill` but understands
    :term:`utf-8` strings and doesn't screw up lists/blocks/etc.

    Use :func:`kitchen.text.display.fill` instead.
    '''
    warnings.warn('kitchen.text.utf8.utf8_text_fill is deprecated.  Use'
        ' kitchen.text.display.fill instead',
        DeprecationWarning, stacklevel=2)
    # This assumes that all args. are utf8.
    return fill(text, *args, **kwargs)

def _utf8_width_le(width, *args):
    '''**Deprecated** Convert the arguments to unicode and use
    :func:`kitchen.text.display._textual_width_le` instead.
    '''
    warnings.warn('kitchen.text.utf8._utf8_width_le is deprecated.  Use'
        ' kitchen.text.display._textual_width_le instead',
        DeprecationWarning, stacklevel=2)
    # This assumes that all args. are utf8.
    return _textual_width_le(width, to_unicode(''.join(args)))

__all__ = ('utf8_text_fill', 'utf8_text_wrap', 'utf8_valid', 'utf8_width',
        'utf8_width_chop', 'utf8_width_fill')



# All versions of python3 include defaultdict.  This module just exists for
# backwards compatibility
#
import warnings

warnings.warn('In python3, kitchen.pycompat25.collections is deprecated'
        ' If you do not need to maintain compatibility with python less'
        ' than 2.5 use collections from the stdlib instead.',
        PendingDeprecationWarning, stacklevel=2)

from collections import defaultdict as _d

class defaultdict(_d):
    '''*Deprecated*.  See help(collections.defaultdict) for usage'''
    def __init__(self, *args, **kwargs):
        warnings.warn('In python3, kitchen.pycompat25.collections.defaultdict'
                ' is deprecated.  If you do not need to maintain compatibility'
                ' with python less than 2.5 use collections.defaultdict from'
                ' the stdlib instead.',
                PendingDeprecationWarning, stacklevel=2)
        super(defaultdict, self).__init__(*args, **kwargs)

__all__ = ('defaultdict',)



# -*- coding: utf-8 -*-
#
# Base class for testing unicode and utf8 functions.  This holds data that's
# useful for making tests

import re

from kitchen.text.converters import to_bytes
from kitchen.text import misc

class UnicodeTestData(object):
    u_empty_string = u''
    b_empty_string = ''
    # This should encode fine -- sanity check
    u_ascii = u'the quick brown fox jumped over the lazy dog'
    b_ascii = 'the quick brown fox jumped over the lazy dog'

    # First challenge -- what happens with latin-1 characters
    u_spanish = u'El veloz murciélago saltó sobre el perro perezoso.'
    # utf8 and latin1 both support these chars so no mangling
    utf8_spanish = u_spanish.encode('utf-8')
    latin1_spanish = u_spanish.encode('latin1')

    # ASCII does not have the accented characters so it mangles
    ascii_mangled_spanish_as_ascii = u_spanish.encode('ascii', 'replace')
    # Attempting to decode using the wrong charset will mangle
    # Note: as a general principle, we do not want to have code that mangles
    # input of one charset and output of the same charset.  We want to avoid
    # things like::
    #   input latin-1, transform to unicode with utf-8, output latin-1.
    u_mangled_spanish_utf8_as_latin1 = unicode(utf8_spanish, encoding='latin1', errors='replace')
    u_mangled_spanish_utf8_as_ascii = unicode(utf8_spanish, encoding='ascii', errors='replace')
    u_mangled_spanish_latin1_as_ascii = unicode(latin1_spanish, encoding='ascii', errors='replace')
    u_mangled_spanish_latin1_as_utf8 = unicode(latin1_spanish, encoding='utf-8', errors='replace')
    ascii_twice_mangled_spanish_latin1_as_utf8_as_ascii = u_mangled_spanish_latin1_as_utf8.encode('ascii', 'replace')
    utf8_mangled_spanish_latin1_as_utf8 = u_mangled_spanish_latin1_as_utf8.encode('utf-8')
    u_spanish_ignore = unicode(latin1_spanish, encoding='utf8', errors='ignore')

    u_japanese = u"速い茶色のキツネが怠惰な犬に'増"
    utf8_japanese = u_japanese.encode('utf8')
    euc_jp_japanese = u_japanese.encode('euc_jp')
    u_mangled_euc_jp_as_latin1 = unicode(euc_jp_japanese, 'latin1')
    u_mangled_euc_jp_as_utf8 = unicode(euc_jp_japanese, 'utf-8', 'replace')
    utf8_mangled_euc_jp_as_latin1 = u_mangled_euc_jp_as_latin1.encode('utf8')
    u_mangled_japanese_utf8_as_latin1 = unicode(utf8_japanese, 'latin1')
    u_mangled_japanese_utf8_as_ascii = u"������������������������������������������'���"
    ascii_mangled_japanese_replace_as_latin1 = "??????????????'?"
    latin1_mangled_japanese_replace_as_latin1 = "??????????????'?"

    u_mixed = u'く ku ら ra と to み mi'
    utf8_mixed = u_mixed.encode('utf8')
    utf8_ku = u_mixed[0].encode('utf8')
    utf8_ra = u_mixed[2].encode('utf8')
    utf8_to = u_mixed[4].encode('utf8')
    utf8_mi = u_mixed[6].encode('utf8')

    u_mixed_replace = u'\ufffd ku \ufffd ra \ufffd to \ufffd mi'
    u_mixed_ignore = u' ku  ra  to  mi'
    latin1_mixed_replace = '? ku ? ra ? to ? mi'
    latin1_mixed_ignore = ' ku  ra  to  mi'

    u_entity = u'Test: <"&"> – ' + u_japanese + u'é'
    utf8_entity = u_entity.encode('utf8')
    u_entity_escape = u'Test: &lt;&quot;&amp;&quot;&gt; &ndash; ' + unicode(u_japanese.encode('ascii', 'xmlcharrefreplace'), 'ascii') + u'&#xe9;'
    utf8_entity_escape = 'Test: &lt;"&amp;"&gt; – 速い茶色のキツネが怠惰な犬に\'増é'
    utf8_attrib_escape = 'Test: &lt;&quot;&amp;&quot;&gt; – 速い茶色のキツネが怠惰な犬に\'増é'
    ascii_entity_escape = ('Test: <"&"> '.replace('&', '&amp;', 1).replace('<', '&lt;').replace('>', '&gt;')) + (u'– ' + u_japanese + u'é').encode('ascii', 'xmlcharrefreplace')
    ascii_attrib_escape = ('Test: <"&"> '.replace('&', '&amp;', 1).replace('<', '&lt;').replace('>', '&gt;').replace('"', '&quot;')) + (u'– ' + u_japanese + u'é').encode('ascii', 'xmlcharrefreplace')

    b_byte_chars = ' '.join(map(chr, range(0, 256)))
    b_byte_encoded = 'ACABIAIgAyAEIAUgBiAHIAggCSAKIAsgDCANIA4gDyAQIBEgEiATIBQgFSAWIBcgGCAZIBogGyAcIB0gHiAfICAgISAiICMgJCAlICYgJyAoICkgKiArICwgLSAuIC8gMCAxIDIgMyA0IDUgNiA3IDggOSA6IDsgPCA9ID4gPyBAIEEgQiBDIEQgRSBGIEcgSCBJIEogSyBMIE0gTiBPIFAgUSBSIFMgVCBVIFYgVyBYIFkgWiBbIFwgXSBeIF8gYCBhIGIgYyBkIGUgZiBnIGggaSBqIGsgbCBtIG4gbyBwIHEgciBzIHQgdSB2IHcgeCB5IHogeyB8IH0gfiB/IIAggSCCIIMghCCFIIYghyCIIIkgiiCLIIwgjSCOII8gkCCRIJIgkyCUIJUgliCXIJggmSCaIJsgnCCdIJ4gnyCgIKEgoiCjIKQgpSCmIKcgqCCpIKogqyCsIK0griCvILAgsSCyILMgtCC1ILYgtyC4ILkguiC7ILwgvSC+IL8gwCDBIMIgwyDEIMUgxiDHIMggySDKIMsgzCDNIM4gzyDQINEg0iDTINQg1SDWINcg2CDZINog2yDcIN0g3iDfIOAg4SDiIOMg5CDlIOYg5yDoIOkg6iDrIOwg7SDuIO8g8CDxIPIg8yD0IPUg9iD3IPgg+SD6IPsg/CD9IP4g/w=='

    repr_re = re.compile('^<[^ ]*\.([^.]+) object at .*>$')

    u_paragraph = u'''ConfigObj is a simple but powerful config file reader and writer: an ini file
round tripper. Its main feature is that it is very easy to use, with a
straightforward programmer's interface and a simple syntax for config files.
It has lots of other features though:



    * Nested sections (subsections), to any level
    * List values
    * Multiple line values
    * String interpolation (substitution)
    * Integrated with a powerful validation system
          o including automatic type checking/conversion
          o repeated sections
          o and allowing default values
    * All comments in the file are preserved
    * The order of keys/sections is preserved
    * No external dependencies
    * Full Unicode support
    * A powerful unrepr mode for storing basic datatypes
'''
    utf8_paragraph = u_paragraph.encode('utf-8', 'replace')
    u_paragraph_out = [u'ConfigObj is a simple but powerful config file reader and writer: an',
u'ini file round tripper. Its main feature is that it is very easy to',
u"use, with a straightforward programmer's interface and a simple syntax",
u'for config files. It has lots of other features though:',
u'',
u'',
u'',
u'    * Nested sections (subsections), to any level',
u'    * List values',
u'    * Multiple line values',
u'    * String interpolation (substitution)',
u'    * Integrated with a powerful validation system',
u'          o including automatic type checking/conversion',
u'          o repeated sections',
u'          o and allowing default values',
u'    * All comments in the file are preserved',
u'    * The order of keys/sections is preserved',
u'    * No external dependencies',
u'    * Full Unicode support',
u'    * A powerful unrepr mode for storing basic datatypes']

    utf8_paragraph_out = [line.encode('utf-8', 'replace') for line in u_paragraph_out]

    u_mixed_para = u'くらとみ kuratomi ' * 5
    utf8_mixed_para = u_mixed_para.encode('utf8')
    u_mixed_para_out = [u'くらとみ kuratomi くらとみ kuratomi くらとみ kuratomi くらとみ',
            u'kuratomi くらとみ kuratomi']
    u_mixed_para_57_initial_subsequent_out = [u'    くらとみ kuratomi くらとみ kuratomi くらとみ kuratomi',
        u'----くらとみ kuratomi くらとみ kuratomi']
    utf8_mixed_para_out = map(to_bytes, u_mixed_para_out)
    utf8_mixed_para_57_initial_subsequent_out = map(to_bytes, u_mixed_para_57_initial_subsequent_out)

    u_ascii_chars = u' '.join(map(unichr, range(0, 256)))
    u_ascii_no_ctrl = u''.join([c for c in u_ascii_chars if ord(c) not in misc._CONTROL_CODES])
    u_ascii_ctrl_replace = u_ascii_chars.translate(dict([(c, u'?') for c in misc._CONTROL_CODES]))
    utf8_ascii_chars = u_ascii_chars.encode('utf8')

    # These are present in the test catalog as msgids or values
    u_lemon = u'1 lemon'
    utf8_lemon = u_lemon.encode('utf-8')
    latin1_lemon = u_lemon.encode('latin-1')

    u_lemons = u'4 lemons'
    utf8_lemons = u_lemons.encode('utf-8')
    latin1_lemons = u_lemons.encode('latin-1')

    u_limao = u'一 limão'
    utf8_limao = u_limao.encode('utf-8')
    latin1_limao = u_limao.encode('latin-1', 'replace')

    u_limoes = u'四 limões'
    utf8_limoes = u_limoes.encode('utf-8')
    latin1_limoes = u_limoes.encode('latin-1', 'replace')

    u_not_in_catalog = u'café not matched in catalogs'
    utf8_not_in_catalog = u_not_in_catalog.encode('utf-8')
    latin1_not_in_catalog = u_not_in_catalog.encode('latin-1')

    u_kitchen = u'kitchen sink'
    utf8_kitchen = u_kitchen.encode('utf-8')
    latin1_kitchen = u_kitchen.encode('latin-1')

    u_pt_kitchen = u'pia da cozinha'
    utf8_pt_kitchen = u_pt_kitchen.encode('utf-8')
    latin1_pt_kitchen = u_pt_kitchen.encode('latin-1')

    u_kuratomi = u'Kuratomi'
    utf8_kuratomi = u_kuratomi.encode('utf-8')
    latin1_kuratomi = u_kuratomi.encode('latin-1')

    u_ja_kuratomi = u'くらとみ'
    utf8_ja_kuratomi = u_ja_kuratomi.encode('utf-8')
    latin1_ja_kuratomi = u_ja_kuratomi.encode('latin-1', 'replace')

    u_in_fallback = u'Only café in fallback'
    utf8_in_fallback = u_in_fallback.encode('utf-8')
    latin1_in_fallback = u_in_fallback.encode('latin-1')

    u_yes_in_fallback = u'Yes, only café in fallback'
    utf8_yes_in_fallback = u_yes_in_fallback.encode('utf-8')
    latin1_yes_in_fallback = u_yes_in_fallback.encode('latin-1')



# -*- coding: utf-8 -*-
#
import unittest
from nose import tools

from kitchen.text.exceptions import ControlCharError

from kitchen.text import display

import base_classes

class TestDisplay(base_classes.UnicodeTestData, unittest.TestCase):

    def test_internal_interval_bisearch(self):
        '''Test that we can find things in an interval table'''
        table = ((0, 3), (5, 7), (9, 10))
        tools.assert_true(display._interval_bisearch(0, table))
        tools.assert_true(display._interval_bisearch(1, table))
        tools.assert_true(display._interval_bisearch(2, table))
        tools.assert_true(display._interval_bisearch(3, table))
        tools.assert_true(display._interval_bisearch(5, table))
        tools.assert_true(display._interval_bisearch(6, table))
        tools.assert_true(display._interval_bisearch(7, table))
        tools.assert_true(display._interval_bisearch(9, table))
        tools.assert_true(display._interval_bisearch(10, table))
        tools.assert_false(display._interval_bisearch(-1, table))
        tools.assert_false(display._interval_bisearch(4, table))
        tools.assert_false(display._interval_bisearch(8, table))
        tools.assert_false(display._interval_bisearch(11, table))

    def test_internal_generate_combining_table(self):
        '''Test that the combining table we generate is equal to or a subseet of what's in the current table

        If we assert it can mean one of two things:

        1. The code is broken
        2. The table we have is out of date.
        '''
        old_table = display._COMBINING
        new_table = display._generate_combining_table()
        for interval in new_table:
            if interval[0] == interval[1]:
                tools.assert_true(display._interval_bisearch(interval[0], old_table))
            else:
                for codepoint in xrange(interval[0], interval[1] + 1):
                    tools.assert_true(display._interval_bisearch(interval[0], old_table))

    def test_internal_ucp_width(self):
        '''Test that ucp_width returns proper width for characters'''
        for codepoint in xrange(0, 0xFFFFF + 1):
            if codepoint < 32 or (codepoint < 0xa0 and codepoint >= 0x7f):
                # With strict on, we should raise an error
                tools.assert_raises(ControlCharError, display._ucp_width, codepoint, 'strict')

                if codepoint in (0x08, 0x1b, 0x7f, 0x94):
                    # Backspace, delete, clear delete remove one char
                    tools.eq_(display._ucp_width(codepoint), -1)
                else:
                    # Everything else returns 0
                    tools.eq_(display._ucp_width(codepoint), 0)
            elif display._interval_bisearch(codepoint, display._COMBINING):
                # Combining character
                tools.eq_(display._ucp_width(codepoint), 0)
            elif (codepoint >= 0x1100 and
                    (codepoint <= 0x115f or                     # Hangul Jamo init. consonants
                        codepoint == 0x2329 or codepoint == 0x232a or
                        (codepoint >= 0x2e80 and codepoint <= 0xa4cf and
                            codepoint != 0x303f) or                   # CJK ... Yi
                        (codepoint >= 0xac00 and codepoint <= 0xd7a3) or # Hangul Syllables
                        (codepoint >= 0xf900 and codepoint <= 0xfaff) or # CJK Compatibility Ideographs
                        (codepoint >= 0xfe10 and codepoint <= 0xfe19) or # Vertical forms
                        (codepoint >= 0xfe30 and codepoint <= 0xfe6f) or # CJK Compatibility Forms
                        (codepoint >= 0xff00 and codepoint <= 0xff60) or # Fullwidth Forms
                        (codepoint >= 0xffe0 and codepoint <= 0xffe6) or
                        (codepoint >= 0x20000 and codepoint <= 0x2fffd) or
                        (codepoint >= 0x30000 and codepoint <= 0x3fffd))):
                tools.eq_(display._ucp_width(codepoint), 2)
            else:
                tools.eq_(display._ucp_width(codepoint), 1)

    def test_textual_width(self):
        '''Test that we find the proper number of spaces that a utf8 string will consume'''
        tools.eq_(display.textual_width(self.u_japanese), 31)
        tools.eq_(display.textual_width(self.u_spanish), 50)
        tools.eq_(display.textual_width(self.u_mixed), 23)

    def test_textual_width_chop(self):
        '''utf8_width_chop with byte strings'''
        tools.eq_(display.textual_width_chop(self.u_mixed, 1000), self.u_mixed)
        tools.eq_(display.textual_width_chop(self.u_mixed, 23), self.u_mixed)
        tools.eq_(display.textual_width_chop(self.u_mixed, 22), self.u_mixed[:-1])
        tools.eq_(display.textual_width_chop(self.u_mixed, 19), self.u_mixed[:-4])
        tools.eq_(display.textual_width_chop(self.u_mixed, 1), u'')
        tools.eq_(display.textual_width_chop(self.u_mixed, 2), self.u_mixed[0])
        tools.eq_(display.textual_width_chop(self.u_mixed, 3), self.u_mixed[:2])
        tools.eq_(display.textual_width_chop(self.u_mixed, 4), self.u_mixed[:3])
        tools.eq_(display.textual_width_chop(self.u_mixed, 5), self.u_mixed[:4])
        tools.eq_(display.textual_width_chop(self.u_mixed, 6), self.u_mixed[:5])
        tools.eq_(display.textual_width_chop(self.u_mixed, 7), self.u_mixed[:5])
        tools.eq_(display.textual_width_chop(self.u_mixed, 8), self.u_mixed[:6])
        tools.eq_(display.textual_width_chop(self.u_mixed, 9), self.u_mixed[:7])
        tools.eq_(display.textual_width_chop(self.u_mixed, 10), self.u_mixed[:8])
        tools.eq_(display.textual_width_chop(self.u_mixed, 11), self.u_mixed[:9])
        tools.eq_(display.textual_width_chop(self.u_mixed, 12), self.u_mixed[:10])
        tools.eq_(display.textual_width_chop(self.u_mixed, 13), self.u_mixed[:10])
        tools.eq_(display.textual_width_chop(self.u_mixed, 14), self.u_mixed[:11])
        tools.eq_(display.textual_width_chop(self.u_mixed, 15), self.u_mixed[:12])
        tools.eq_(display.textual_width_chop(self.u_mixed, 16), self.u_mixed[:13])
        tools.eq_(display.textual_width_chop(self.u_mixed, 17), self.u_mixed[:14])
        tools.eq_(display.textual_width_chop(self.u_mixed, 18), self.u_mixed[:15])
        tools.eq_(display.textual_width_chop(self.u_mixed, 19), self.u_mixed[:15])
        tools.eq_(display.textual_width_chop(self.u_mixed, 20), self.u_mixed[:16])
        tools.eq_(display.textual_width_chop(self.u_mixed, 21), self.u_mixed[:17])

    def test_textual_width_fill(self):
        '''Pad a utf8 string'''
        tools.eq_(display.textual_width_fill(self.u_mixed, 1), self.u_mixed)
        tools.eq_(display.textual_width_fill(self.u_mixed, 25), self.u_mixed + u'  ')
        tools.eq_(display.textual_width_fill(self.u_mixed, 25, left=False), u'  ' + self.u_mixed)
        tools.eq_(display.textual_width_fill(self.u_mixed, 25, chop=18), self.u_mixed[:-4] + u'       ')
        tools.eq_(display.textual_width_fill(self.u_mixed, 25, chop=18, prefix=self.u_spanish, suffix=self.u_spanish), self.u_spanish + self.u_mixed[:-4] + self.u_spanish + u'       ')
        tools.eq_(display.textual_width_fill(self.u_mixed, 25, chop=18), self.u_mixed[:-4] + u'       ')
        tools.eq_(display.textual_width_fill(self.u_mixed, 25, chop=18, prefix=self.u_spanish, suffix=self.u_spanish), self.u_spanish + self.u_mixed[:-4] + self.u_spanish + u'       ')

    def test_internal_textual_width_le(self):
        test_data = ''.join([self.u_mixed, self.u_spanish])
        tw = display.textual_width(test_data)
        tools.eq_(display._textual_width_le(68, self.u_mixed, self.u_spanish), (tw <= 68))
        tools.eq_(display._textual_width_le(69, self.u_mixed, self.u_spanish), (tw <= 69))
        tools.eq_(display._textual_width_le(137, self.u_mixed, self.u_spanish), (tw <= 137))
        tools.eq_(display._textual_width_le(138, self.u_mixed, self.u_spanish), (tw <= 138))
        tools.eq_(display._textual_width_le(78, self.u_mixed, self.u_spanish), (tw <= 78))
        tools.eq_(display._textual_width_le(79, self.u_mixed, self.u_spanish), (tw <= 79))

    def test_wrap(self):
        '''Test that text wrapping works'''
        tools.eq_(display.wrap(self.u_mixed), [self.u_mixed])
        tools.eq_(display.wrap(self.u_paragraph), self.u_paragraph_out)
        tools.eq_(display.wrap(self.utf8_paragraph), self.u_paragraph_out)
        tools.eq_(display.wrap(self.u_mixed_para), self.u_mixed_para_out)
        tools.eq_(display.wrap(self.u_mixed_para, width=57,
            initial_indent='    ', subsequent_indent='----'),
            self.u_mixed_para_57_initial_subsequent_out)

    def test_fill(self):
        tools.eq_(display.fill(self.u_paragraph), u'\n'.join(self.u_paragraph_out))
        tools.eq_(display.fill(self.utf8_paragraph), u'\n'.join(self.u_paragraph_out))
        tools.eq_(display.fill(self.u_mixed_para), u'\n'.join(self.u_mixed_para_out))
        tools.eq_(display.fill(self.u_mixed_para, width=57,
            initial_indent='    ', subsequent_indent='----'),
            u'\n'.join(self.u_mixed_para_57_initial_subsequent_out))

    def test_byte_string_textual_width_fill(self):
        tools.eq_(display.byte_string_textual_width_fill(self.utf8_mixed, 1), self.utf8_mixed)
        tools.eq_(display.byte_string_textual_width_fill(self.utf8_mixed, 25), self.utf8_mixed + '  ')
        tools.eq_(display.byte_string_textual_width_fill(self.utf8_mixed, 25, left=False), '  ' + self.utf8_mixed)
        tools.eq_(display.byte_string_textual_width_fill(self.utf8_mixed, 25, chop=18), self.u_mixed[:-4].encode('utf8') + '       ')
        tools.eq_(display.byte_string_textual_width_fill(self.utf8_mixed, 25, chop=18, prefix=self.utf8_spanish, suffix=self.utf8_spanish), self.utf8_spanish + self.u_mixed[:-4].encode('utf8') + self.utf8_spanish + '       ')
        tools.eq_(display.byte_string_textual_width_fill(self.utf8_mixed, 25, chop=18), self.u_mixed[:-4].encode('utf8') + '       ')
        tools.eq_(display.byte_string_textual_width_fill(self.utf8_mixed, 25, chop=18, prefix=self.utf8_spanish, suffix=self.utf8_spanish), self.utf8_spanish + self.u_mixed[:-4].encode('utf8') + self.utf8_spanish + '       ')



# :W0401, W0611, W0614: Rather than have two versions of subprocess, we import
#   the python2.7 version here as well
#pylint:disable-msg=W0401,W0611,W0614
from kitchen.pycompat27.subprocess import *
from kitchen.pycompat27.subprocess import __all__



#!/usr/bin/python -tt
# -*- coding: utf-8 -*-

from distutils.command.sdist import sdist as _sdist
import glob
import os
import sys

if sys.version_info[0] == 2:
    source_dir = 'kitchen2'
    packages = [
        'kitchen',
        'kitchen.versioning',
        'kitchen.i18n',
        'kitchen.iterutils',
        'kitchen.collections',
        'kitchen.text',
        'kitchen.pycompat24',
        'kitchen.pycompat24.base64',
        'kitchen.pycompat24.sets',
        'kitchen.pycompat25',
        'kitchen.pycompat25.collections',
        'kitchen.pycompat27',
        'kitchen.pycompat27.subprocess',
    ]
elif sys.version_info[0] == 3:
    source_dir = 'kitchen3'
    packages = [
        'kitchen',
        'kitchen.versioning',
        'kitchen.i18n',
        'kitchen.iterutils',
        'kitchen.collections',
        'kitchen.text',
        'kitchen.pycompat24',
        'kitchen.pycompat24.base64',
        'kitchen.pycompat24.sets',
        'kitchen.pycompat25',
        'kitchen.pycompat25.collections',
        'kitchen.pycompat27',
        'kitchen.pycompat27.subprocess',
    ]
else:
    raise NotImplementedError("Python version unsupported %r" % sys.version)

sys.path.append(os.path.abspath(source_dir))

# Now that we have modified sys.path, these imports will pull in either the py3
# version or the py2 version.
import kitchen.release

import releaseutils

from setuptools import setup

# Override sdist command to compile the message catalogs as well
class Sdist(_sdist, object):
    def run(self):
        releaseutils.main()
        data_files = []
        for langfile in filter(os.path.isfile, glob.glob('locale/*/*/*.mo')):
            data_files.append((os.path.dirname(langfile), [langfile]))
        if self.distribution.data_files and \
                hasattr(self.distribution.data_files, 'extend'):
            self.distribution.data_files.extend(data_files)
        else:
            self.distribution.data_files = data_files
        super(Sdist, self).run()


setup(name='kitchen',
      version=str(kitchen.release.__version__),
      description=kitchen.release.DESCRIPTION,
      long_description=kitchen.release.LONG_DESCRIPTION,
      author=kitchen.release.AUTHOR,
      author_email=kitchen.release.EMAIL,
      maintainer='Toshio Kuratomi',
      maintainer_email='toshio@fedoraproject.org',
      license=kitchen.release.LICENSE,
      url=kitchen.release.URL,
      cmdclass={'sdist': Sdist},
      keywords='Useful Small Code Snippets',
      classifiers=[
            'Development Status :: 4 - Beta',
            'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
            'Operating System :: OS Independent',
            'Programming Language :: Python :: 2',
            'Programming Language :: Python :: 2.4',
            'Programming Language :: Python :: 2.5',
            'Programming Language :: Python :: 2.6',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.3',
            'Programming Language :: Python :: 3.4',
            'Topic :: Software Development :: Internationalization',
            'Topic :: Software Development :: Libraries :: Python Modules',
            'Topic :: Text Processing :: General',
          ],
      packages=packages,
      package_dir={'': source_dir},
      data_files=[],
)



'''
------------
Kitchen.text
------------

Kitchen.text contains functions for manipulating text in python.

This includes things like converting between byte strings and unicode,
and displaying text on the screen.
'''

from kitchen.versioning import version_tuple_to_string

__version_info__ = ((2, 2, 0),)
__version__ = version_tuple_to_string(__version_info__)

__all__ = ('converters', 'exceptions', 'misc',)



'''
The :mod:`kitchen.pycompat25` module contains implementations of functionality
introduced in python-2.5.
'''
import warnings

warnings.warn('In python3, kitchen.pycompat25 is deprecated because all'
        ' functionality in this module is found in the python3 stdlib.'
        '  If your code doesn\'t need to maintain compatibility with'
        ' python less than 2.5, use collections.defaultdict from the python3'
        ' stdlib.',
        PendingDeprecationWarning, stacklevel=2)

from kitchen.versioning import version_tuple_to_string

__version_info__ = ((1, 0, 0),)
__version__ = version_tuple_to_string(__version_info__)


__all__ = ('collections',)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2011 Red Hat, Inc
#
# kitchen is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# kitchen is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with kitchen; if not, see <http://www.gnu.org/licenses/>
#
# Authors:
#   Toshio Kuratomi <toshio@fedoraproject.org>
'''
----------------------------
PEP-386 compliant versioning
----------------------------

:pep:`386` defines a standard format for version strings.  This module
contains a function for creating strings in that format.
'''
__version_info__ = ((1, 0, 0),)

import itertools

def version_tuple_to_string(version_info):
    '''Return a :pep:`386` version string from a :pep:`386` style version tuple

    :arg version_info: Nested set of tuples that describes the version.  See
        below for an example.
    :returns: a version string

    This function implements just enough of :pep:`386` to satisfy our needs.
    :pep:`386` defines a standard format for version strings and refers to
    a function that will be merged into the |stdlib|_ that transforms a tuple
    of version information into a standard version string.  This function is
    an implementation of that function.  Once that function becomes available
    in the |stdlib|_ we will start using it and deprecate this function.

    :attr:`version_info` takes the form that :pep:`386`'s
    :func:`NormalizedVersion.from_parts` uses::

        ((Major, Minor, [Micros]), [(Alpha/Beta/rc marker, version)],
            [(post/dev marker, version)])

        Ex: ((1, 0, 0), ('a', 2), ('dev', 3456))

    It generates a :pep:`386` compliant version string::

        N.N[.N]+[{a|b|c|rc}N[.N]+][.postN][.devN]

        Ex: 1.0.0a2.dev3456

    .. warning:: This function does next to no error checking.  It's up to the
        person defining the version tuple to make sure that the values make
        sense.  If the :pep:`386` compliant version parser doesn't get
        released soon we'll look at making this function check that the
        version tuple makes sense before transforming it into a string.

    It's recommended that you use this function to keep
    a :data:`__version_info__` tuple and :data:`__version__` string in your
    modules.  Why do we need both a tuple and a string?  The string is often
    useful for putting into human readable locations like release
    announcements, version strings in tarballs, etc.  Meanwhile the tuple is
    very easy for a computer to compare. For example, kitchen sets up its
    version information like this::

        from kitchen.versioning import version_tuple_to_string
        __version_info__ = ((0, 2, 1),)
        __version__ = version_tuple_to_string(__version_info__)

    Other programs that depend on a kitchen version between 0.2.1 and 0.3.0
    can find whether the present version is okay with code like this::

        from kitchen import __version_info__, __version__
        if __version_info__ < ((0, 2, 1),) or __version_info__ >= ((0, 3, 0),):
            print 'kitchen is present but not at the right version.'
            print 'We need at least version 0.2.1 and less than 0.3.0'
            print 'Currently found: kitchen-%s' % __version__
    '''
    ver_components = []
    for values in version_info:
        if isinstance(values[0], int):
            ver_components.append('.'.join(itertools.imap(str, values)))
        else:
            modifier = values[0]
            if isinstance(values[0], unicode):
                modifier = values[0].encode('ascii')

            if  modifier in ('a', 'b', 'c', 'rc'):
                ver_components.append('%s%s' % (modifier,
                    '.'.join(itertools.imap(str, values[1:])) or '0'))
            else:
                ver_components.append('.%s%s' % (modifier,
                    str(values[1])))
    return unicode(''.join(ver_components), 'ascii')


__version__ = version_tuple_to_string(__version_info__)

__all__ = ('version_tuple_to_string',)



##
# Transcribed from http://code.activestate.com/recipes/523034/ on May 1, 2009
# by Jef Spaleta This code provides an emulation for the defaultdict
# functionality introduced in python 2.5's collection module
#
# Changes from the original:
# * Change the return value from __reduce__ to use iteritems() to prevent
#   a segfault when pickling. (Jef Spaleta)
# * Change how we setup the module to use collections.defaultdict by default
#   (Toshio Kuratomi)
#
# Copyright (c) 2007 Justin Kirtland
#
# PYTHON SOFTWARE FOUNDATION LICENSE VERSION 2
# --------------------------------------------
#
# 1. This LICENSE AGREEMENT is between the Python Software Foundation ("PSF"),
#    and the Individual or Organization ("Licensee") accessing and otherwise
#    using this software ("Python") in source or binary form and its
#    associated documentation.
#
# 2. Subject to the terms and conditions of this License Agreement, PSF hereby
#    grants Licensee a nonexclusive, royalty-free, world-wide license to
#    reproduce, analyze, test, perform and/or display publicly, prepare
#    derivative works, distribute, and otherwise use Python alone or in any
#    derivative version, provided, however, that PSF's License Agreement and
#    PSF's notice of copyright, i.e., "Copyright (c) 2001, 2002, 2003, 2004,
#    2005, 2006 Python Software Foundation; All Rights Reserved" are retained
#    in Python alone or in any derivative version prepared by Licensee.
#
# 3. In the event Licensee prepares a derivative work that is based on or
#    incorporates Python or any part thereof, and wants to make the derivative
#    work available to others as provided herein, then Licensee hereby agrees
#    to include in any such work a brief summary of the changes made to
#    Python.
#
# 4. PSF is making Python available to Licensee on an "AS IS" basis.  PSF
#    MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.  BY WAY OF
#    EXAMPLE, BUT NOT LIMITATION, PSF MAKES NO AND DISCLAIMS ANY
#    REPRESENTATION OR WARRANTY OF MERCHANTABILITY OR FITNESS FOR ANY
#    PARTICULAR PURPOSE OR THAT THE USE OF PYTHON WILL NOT INFRINGE ANY THIRD
#    PARTY RIGHTS.
#
# 5. PSF SHALL NOT BE LIABLE TO LICENSEE OR ANY OTHER USERS OF PYTHON FOR ANY
#    INCIDENTAL, SPECIAL, OR CONSEQUENTIAL DAMAGES OR LOSS AS A RESULT OF
#    MODIFYING, DISTRIBUTING, OR OTHERWISE USING PYTHON, OR ANY DERIVATIVE
#    THEREOF, EVEN IF ADVISED OF THE POSSIBILITY THEREOF.
#
# 6. This License Agreement will automatically terminate upon a material
#    breach of its terms and conditions.
#
# 7. Nothing in this License Agreement shall be deemed to create any
#    relationship of agency, partnership, or joint venture between PSF and
#    Licensee.  This License Agreement does not grant permission to use PSF
#    trademarks or trade name in a trademark sense to endorse or promote
#    products or services of Licensee, or any third party.
#
# 8. By copying, installing or otherwise using Python, Licensee agrees to be
#    bound by the terms and conditions of this License Agreement.

'''
-----------
defaultdict
-----------

This is a pure python implementation of defaultdict that is compatible with
the defaultdict class provided by python-2.5 and above.

.. seealso::
    :class:`collections.defaultdict`
        for documentation on this module
'''

# Pylint disabled messages
#
# :C0103: We're defnining a compatible class name therefore we need to match
#   the format of that name.

import types

# :C0103, W0613: We're implementing the python-2.5 defaultdict API so
#   we have to use the same names as python.
# :C0111: We point people at the stdlib API docs for defaultdict rather than
#   reproduce it here.
#pylint:disable-msg=C0103,W0613,C0111

class defaultdict(dict):
    def __init__(self, default_factory=None, *args, **kwargs):
        if (default_factory is not None and
                not hasattr(default_factory, '__call__')):
            raise TypeError('First argument must be callable')
        dict.__init__(self, *args, **kwargs)
        self.default_factory = default_factory

    def __getitem__(self, key):
        try:
            return dict.__getitem__(self, key)
        except KeyError:
            return self.__missing__(key)

    def __missing__(self, key):
        if self.default_factory is None:
            raise KeyError(key)
        self[key] = value = self.default_factory()
        return value

    def __reduce__(self):
        if self.default_factory is None:
            args = tuple()
        else:
            args = self.default_factory,
        return type(self), args, None, None, self.iteritems()

    def copy(self):
        return self.__copy__()

    def __copy__(self):
        return type(self)(self.default_factory, self)

    def __deepcopy__(self, memo):
        import copy
        return type(self)(self.default_factory,
                          copy.deepcopy(self.items()))
    def __repr__(self):
        # Note: Have to use "is not None"  otherwise we get an infinite
        # recursion
        if isinstance(self.default_factory, types.MethodType) \
                and self.default_factory.im_self is not None \
                and issubclass(self.default_factory.im_class, defaultdict):
            defrepr = '<bound method sub._factory of defaultdict(...'
        else:
            defrepr = repr(self.default_factory)
        return 'defaultdict(%s, %s)' % (defrepr, dict.__repr__(self))

__all__ = ('defaultdict',)
